@@ -32,8 +32,8 @@
 
 HWGame::HWGame(GameUIConfig * config, GameCFGWidget * gamecfg, QString ammo, TeamSelWidget* pTeamSelWidget) :
   TCPBase(true),
-  m_pTeamSelWidget(pTeamSelWidget),
-  ammostr(ammo)
+  ammostr(ammo),
+  m_pTeamSelWidget(pTeamSelWidget)
 {
 	this->config = config;
 	this->gamecfg = gamecfg;
@@ -48,6 +48,7 @@ HWGame::~HWGame()
 void HWGame::onClientDisconnect()
 {
 	switch (gameType) {
+		case gtDemo: break;
 		case gtNet:
 			emit HaveRecord(true, demo);
 			break;
@@ -103,22 +104,28 @@ void HWGame::SendQuickConfig()
 			.arg((Themes->size() > 0) ? Themes->at(rand() % Themes->size()) : "steel"));
 	HWProto::addStringToBuffer(teamscfg, "eseed " + QUuid::createUuid().toString());
 
-	HWTeam team1(0);
-	team1.difficulty = 0;
-	team1.teamColor = *color1;
-	team1.numHedgehogs = 4;
-	HWProto::addStringListToBuffer(teamscfg,
-			team1.TeamGameConfig(100));
+	HWNamegen namegen;
 
-	HWTeam team2(2);
-	team2.difficulty = 4;
-	team2.teamColor = *color2;
-	team2.numHedgehogs = 4;
+	HWTeam * team1;
+	team1 = new HWTeam;
+	team1->difficulty = 0;
+	team1->teamColor = *color1;
+	team1->numHedgehogs = 4;
+	namegen.TeamRandomNames(team1,TRUE);
 	HWProto::addStringListToBuffer(teamscfg,
-			team2.TeamGameConfig(100));
+			team1->TeamGameConfig(100));
 
-	HWProto::addStringToBuffer(teamscfg, *cDefaultAmmoStore);
-	HWProto::addStringToBuffer(teamscfg, *cDefaultAmmoStore);
+	HWTeam * team2;
+	team2 = new HWTeam;
+	team2->difficulty = 4;
+	team2->teamColor = *color2;
+	team2->numHedgehogs = 4;
+	namegen.TeamRandomNames(team2,TRUE);
+	HWProto::addStringListToBuffer(teamscfg,
+			team2->TeamGameConfig(100));
+
+	HWProto::addStringToBuffer(teamscfg, "eammstore " + *cDefaultAmmoStore);
+	HWProto::addStringToBuffer(teamscfg, "eammstore " + *cDefaultAmmoStore);
 	RawSendIPC(teamscfg);
 }
 
@@ -127,7 +134,7 @@ void HWGame::SendTrainingConfig()
 	QByteArray traincfg;
 	HWProto::addStringToBuffer(traincfg, "TL");
 
-	HWTeam team1(0);
+	HWTeam team1;
 	team1.difficulty = 0;
 	team1.teamColor = *color1;
 	team1.numHedgehogs = 1;
@@ -271,13 +278,14 @@ QStringList HWGame::setArguments()
 	arguments << (config->vid_Fullscreen() ? "1" : "0");
 	arguments << (config->isSoundEnabled() ? "1" : "0");
 	arguments << tr("en.txt");
-	arguments << "128"; // sound volume
+	arguments << QString::number(config->volume()); // sound volume
 	arguments << QString::number(config->timerInterval());
 	arguments << datadir->absolutePath();
 	arguments << (config->isShowFPSEnabled() ? "1" : "0");
 	arguments << (config->isAltDamageEnabled() ? "1" : "0");
 	arguments << config->netNick().toUtf8().toBase64();
 	arguments << (config->isMusicEnabled() ? "1" : "0");
+	arguments << (config->isReducedQuality() ? "1" : "0");
 	return arguments;
 }
 
