@@ -44,8 +44,11 @@ GameUIConfig::GameUIConfig(HWForm * FormWidgets, const QString & fileName)
 	bool ffscr=value("video/frontendfullscreen", false).toBool();
 	Form->ui.pageOptions->CBFrontendFullscreen->setChecked(ffscr);
 
+	Form->ui.pageOptions->CBReduceQuality->setChecked(value("video/reducequality", false).toBool());
+	Form->ui.pageOptions->CBFrontendEffects->setChecked(frontendEffects);
 	Form->ui.pageOptions->CBEnableSound->setChecked(value("audio/sound", true).toBool());
 	Form->ui.pageOptions->CBEnableMusic->setChecked(value("audio/music", true).toBool());
+	Form->ui.pageOptions->volumeBox->setValue(value("audio/volume", 100).toUInt());
 
 	Form->ui.pageOptions->editNetNick->setText(value("net/nick", QLineEdit::tr("unnamed")).toString());
 
@@ -60,6 +63,7 @@ GameUIConfig::GameUIConfig(HWForm * FormWidgets, const QString & fileName)
 	Form->ui.pageOptions->fpsedit->setValue(value("fps/interval", 27).toUInt());
 
 	Form->ui.pageOptions->CBAltDamage->setChecked(value("misc/altdamage", false).toBool());
+	Form->ui.pageOptions->CBNameWithDate->setChecked(value("misc/appendTimeToRecords", false).toBool());
 
 	depth = QApplication::desktop()->depth();
 	if (depth < 16) depth = 16;
@@ -86,18 +90,24 @@ void GameUIConfig::SaveOptions()
 {
 	setValue("video/resolution", Form->ui.pageOptions->CBResolution->currentText());
 	setValue("video/fullscreen", vid_Fullscreen());
-	bool ffscr=isFrontendFullscreen();
+
+	setValue("video/reducequality", isReducedQuality());
+
+	setValue("video/frontendeffects", isFrontendEffects());
+
+	bool ffscr = isFrontendFullscreen();
 	setValue("video/frontendfullscreen", ffscr);
 	emit frontendFullscreen(ffscr);
 	if (!ffscr) {
 	  setValue("window/width", Form->width());
 	  setValue("window/height", Form->height());
 	} else {
-	  resizeToConfigValues();
+	  //resizeToConfigValues(); // TODO: why this has been made?
 	}
 
 	setValue("audio/sound", isSoundEnabled());
 	setValue("audio/music", isMusicEnabled());
+	setValue("audio/volume", Form->ui.pageOptions->volumeBox->value());
 
 	setValue("net/nick", netNick());
 	setValue("net/ip", *netHost);
@@ -109,6 +119,7 @@ void GameUIConfig::SaveOptions()
 	setValue("fps/interval", Form->ui.pageOptions->fpsedit->value());
 
 	setValue("misc/altdamage", isAltDamageEnabled());
+	setValue("misc/appendTimeToRecords", appendDateTimeToRecordName());
 }
 
 QRect GameUIConfig::vid_Resolution()
@@ -128,6 +139,15 @@ bool GameUIConfig::vid_Fullscreen()
 	return Form->ui.pageOptions->CBFullscreen->isChecked();
 }
 
+bool GameUIConfig::isReducedQuality() const
+{
+  return Form->ui.pageOptions->CBReduceQuality->isChecked();
+}
+bool GameUIConfig::isFrontendEffects() const
+{
+  return Form->ui.pageOptions->CBFrontendEffects->isChecked();
+}
+
 bool GameUIConfig::isFrontendFullscreen() const
 {
   return Form->ui.pageOptions->CBFrontendFullscreen->isChecked();
@@ -150,7 +170,12 @@ bool GameUIConfig::isShowFPSEnabled()
 
 bool GameUIConfig::isAltDamageEnabled()
 {
-	return Form->ui.pageOptions->CBAltDamage->isChecked();;
+	return Form->ui.pageOptions->CBAltDamage->isChecked();
+}
+
+bool GameUIConfig::appendDateTimeToRecordName()
+{
+	return Form->ui.pageOptions->CBNameWithDate->isChecked();
 }
 
 quint8 GameUIConfig::timerInterval()
@@ -167,3 +192,8 @@ QString GameUIConfig::netNick()
 {
 	return Form->ui.pageOptions->editNetNick->text();
 }
+
+quint8 GameUIConfig::volume()
+{
+	return Form->ui.pageOptions->volumeBox->value() * 128 / 100;
+}
