@@ -39,9 +39,13 @@ public:
 	bool isMusicEnabled();
 	bool isShowFPSEnabled();
 	bool isAltDamageEnabled();
+	bool appendDateTimeToRecordName();
+	quint8 volume();
 	quint8 timerInterval();
 	quint8 bitDepth();
 	QString netNick();
+	bool isReducedQuality() const;
+	bool isFrontendEffects() const;
 	bool isFrontendFullscreen() const;
 	void resizeToConfigValues();
 
