@@ -20,6 +20,7 @@
 #include <QDir>
 #include <QStringList>
 #include <QColor>
+#include <QPair>
 
 extern QString * cProtoVer;
 extern QString * cVersionString;
@@ -33,6 +34,7 @@ extern QStringList * Themes;
 extern QStringList * mapList;
 
 extern QString * cDefaultAmmoStore;
+extern QList< QPair<QString, QString> > cDefaultAmmos;
 
 extern QColor * color1;
 extern QColor * color2;
