@@ -1,6 +1,6 @@
 /*
  * Hedgewars, a free turn based strategy game
- * Copyright (c) 2005-2008 Andrey Korotaev <unC0Rr@gmail.com>
+ * Copyright (c) 2005-2009 Andrey Korotaev <unC0Rr@gmail.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -32,14 +32,18 @@
 #include <QAction>
 #include <QTimer>
 #include <QScrollBar>
+#include <QDataWidgetMapper>
+#include <QTableView>
 
 #include "hwform.h"
 #include "game.h"
 #include "team.h"
+#include "namegen.h"
 #include "teamselect.h"
 #include "selectWeapon.h"
 #include "gameuiconfig.h"
 #include "pages.h"
+#include "statsPage.h"
 #include "hwconsts.h"
 #include "newnetclient.h"
 #include "gamecfgwidget.h"
@@ -48,10 +52,19 @@
 #include "chatwidget.h"
 #include "playrecordpage.h"
 #include "input_ip.h"
+#include "ammoSchemeModel.h"
+#include "bgwidget.h"
+
+// I started handing this down to each place it touches, but it was getting ridiculous
+// and this one flag does not warrant a static class
+bool frontendEffects = true;
 
 HWForm::HWForm(QWidget *parent)
   : QMainWindow(parent), pnetserver(0), pRegisterServer(0), editedTeam(0), hwnet(0)
 {
+    QSettings settings(cfgdir->absolutePath() + "/hedgewars.ini", QSettings::IniFormat);
+    frontendEffects = settings.value("video/frontendeffects", true).toBool();
+
 	ui.setupUi(this);
 
 	CustomizePalettes();
@@ -59,6 +72,8 @@ HWForm::HWForm(QWidget *parent)
 	ui.pageOptions->CBResolution->addItems(sdli.getResolutions());
 
 	config = new GameUIConfig(this, cfgdir->absolutePath() + "/hedgewars.ini");
+	
+	namegen = new HWNamegen();
 
 	UpdateTeamsLists();
 	UpdateWeapons();
@@ -66,79 +81,110 @@ HWForm::HWForm(QWidget *parent)
 	connect(config, SIGNAL(frontendFullscreen(bool)), this, SLOT(onFrontendFullscreen(bool)));
 	onFrontendFullscreen(config->isFrontendFullscreen());
 
-	connect(ui.pageMain->BtnSinglePlayer,	SIGNAL(clicked()),	this, SLOT(GoToSinglePlayer()));
-	connect(ui.pageMain->BtnSetup,	SIGNAL(clicked()),	this, SLOT(GoToSetup()));
-	connect(ui.pageMain->BtnNet,	SIGNAL(clicked()),	this, SLOT(GoToNet()));
-	connect(ui.pageMain->BtnInfo,	SIGNAL(clicked()),	this, SLOT(GoToInfo()));
+	connect(ui.pageMain->BtnSinglePlayer, SIGNAL(clicked()), this, SLOT(GoToSinglePlayer()));
+	connect(ui.pageMain->BtnSetup, SIGNAL(clicked()), this, SLOT(GoToSetup()));
+	connect(ui.pageMain->BtnNet, SIGNAL(clicked()), this, SLOT(GoToNetType()));
+	connect(ui.pageMain->BtnInfo, SIGNAL(clicked()), this, SLOT(GoToInfo()));
 	connect(ui.pageMain->BtnExit, SIGNAL(pressed()), this, SLOT(btnExitPressed()));
 	connect(ui.pageMain->BtnExit, SIGNAL(clicked()), this, SLOT(btnExitClicked()));
 
-	connect(ui.pageEditTeam->BtnTeamSave,	SIGNAL(clicked()),	this, SLOT(TeamSave()));
-	connect(ui.pageEditTeam->BtnTeamDiscard,	SIGNAL(clicked()),	this, SLOT(TeamDiscard()));
+	connect(ui.pageEditTeam->BtnTeamSave, SIGNAL(clicked()), this, SLOT(TeamSave()));
+	connect(ui.pageEditTeam->BtnTeamDiscard, SIGNAL(clicked()), this, SLOT(TeamDiscard()));
 
-	connect(ui.pageMultiplayer->BtnBack,	SIGNAL(clicked()),	this, SLOT(GoBack()));
-	connect(ui.pageMultiplayer->BtnStartMPGame,	SIGNAL(clicked()),	this, SLOT(StartMPGame()));
+	connect(ui.pageEditTeam->signalMapper, SIGNAL(mapped(const int &)), this, SLOT(RandomName(const int &)));
+	connect(ui.pageEditTeam->randTeamButton, SIGNAL(clicked()), this, SLOT(RandomNames()));
+
+	connect(ui.pageMultiplayer->BtnBack, SIGNAL(clicked()), this, SLOT(GoBack()));
+	connect(ui.pageMultiplayer->BtnStartMPGame, SIGNAL(clicked()), this, SLOT(StartMPGame()));
 	connect(ui.pageMultiplayer->teamsSelect, SIGNAL(setEnabledGameStart(bool)),
 		ui.pageMultiplayer->BtnStartMPGame, SLOT(setEnabled(bool)));
 	connect(ui.pageMultiplayer->teamsSelect, SIGNAL(SetupClicked()), this, SLOT(IntermediateSetup()));
+	connect(ui.pageMultiplayer->gameCFG, SIGNAL(goToSchemes()), this, SLOT(GoToSchemes()));
+	connect(ui.pageMultiplayer->gameCFG, SIGNAL(goToWeapons(const QString &)), this, SLOT(GoToSelectWeaponSet(const QString &)));
 
-	connect(ui.pagePlayDemo->BtnBack,	SIGNAL(clicked()),	this, SLOT(GoBack()));
-	connect(ui.pagePlayDemo->BtnPlayDemo,	SIGNAL(clicked()),	this, SLOT(PlayDemo()));
-	connect(ui.pagePlayDemo->DemosList,	SIGNAL(doubleClicked (const QModelIndex &)),	this, SLOT(PlayDemo()));
+	connect(ui.pagePlayDemo->BtnBack, SIGNAL(clicked()), this, SLOT(GoBack()));
+	connect(ui.pagePlayDemo->BtnPlayDemo, SIGNAL(clicked()), this, SLOT(PlayDemo()));
+	connect(ui.pagePlayDemo->DemosList, SIGNAL(doubleClicked (const QModelIndex &)), this, SLOT(PlayDemo()));
 
-	connect(ui.pageOptions->BtnBack,	SIGNAL(clicked()),	this, SLOT(GoBack()));
-	connect(ui.pageOptions->BtnNewTeam,	SIGNAL(clicked()),	this, SLOT(NewTeam()));
-	connect(ui.pageOptions->BtnEditTeam,	SIGNAL(clicked()),	this, SLOT(EditTeam()));
-	connect(ui.pageOptions->BtnSaveOptions,	SIGNAL(clicked()),	config, SLOT(SaveOptions()));
-	connect(ui.pageOptions->BtnSaveOptions,	SIGNAL(clicked()),	this, SLOT(GoBack()));
+	connect(ui.pageOptions->BtnBack, SIGNAL(clicked()), this, SLOT(GoBack()));
+	connect(ui.pageOptions->BtnNewTeam, SIGNAL(clicked()), this, SLOT(NewTeam()));
+	connect(ui.pageOptions->BtnEditTeam, SIGNAL(clicked()), this, SLOT(EditTeam()));
+	connect(ui.pageOptions->BtnSaveOptions, SIGNAL(clicked()), config, SLOT(SaveOptions()));
+	connect(ui.pageOptions->BtnSaveOptions, SIGNAL(clicked()), this, SLOT(GoBack()));
 
-	connect(ui.pageOptions->WeaponEdit,	SIGNAL(clicked()),	this, SLOT(GoToSelectWeapon()));
-	connect(ui.pageOptions->WeaponsButt,	SIGNAL(clicked()),	this, SLOT(GoToSelectNewWeapon()));
-	connect(ui.pageSelectWeapon->pWeapons,       SIGNAL(weaponsChanged()), this, SLOT(UpdateWeapons()));
-	connect(ui.pageNetGame->pGameCFG,       SIGNAL(newWeaponsName(const QString&)), this, SLOT(NetWeaponNameChanged(const QString&)));
+	connect(ui.pageOptions->WeaponEdit, SIGNAL(clicked()), this, SLOT(GoToSelectWeapon()));
+	connect(ui.pageOptions->WeaponsButt, SIGNAL(clicked()), this, SLOT(GoToSelectNewWeapon()));
+	connect(ui.pageSelectWeapon->pWeapons, SIGNAL(weaponsChanged()), this, SLOT(UpdateWeapons()));
 
 	connect(ui.pageNet->BtnBack, SIGNAL(clicked()), this, SLOT(GoBack()));
 	connect(ui.pageNet->BtnSpecifyServer, SIGNAL(clicked()), this, SLOT(NetConnect()));
-	connect(ui.pageNet->BtnOfficialServer, SIGNAL(clicked()), this, SLOT(NetConnectOfficialServer()));
 	connect(ui.pageNet->BtnNetSvrStart, SIGNAL(clicked()), this, SLOT(GoToNetServer()));
 	connect(ui.pageNet, SIGNAL(connectClicked(const QString &, quint16)), this, SLOT(NetConnectServer(const QString &, quint16)));
 	
-	connect(ui.pageNetServer->BtnBack,	SIGNAL(clicked()),	this, SLOT(GoBack()));
-	connect(ui.pageNetServer->BtnStart,	SIGNAL(clicked()),	this, SLOT(NetStartServer()));
+	connect(ui.pageNetServer->BtnBack, SIGNAL(clicked()), this, SLOT(GoBack()));
+	connect(ui.pageNetServer->BtnStart, SIGNAL(clicked()), this, SLOT(NetStartServer()));
 
-	connect(ui.pageNetGame->BtnBack,	SIGNAL(clicked()),	this, SLOT(GoBack()));
+	connect(ui.pageNetGame->BtnBack, SIGNAL(clicked()), this, SLOT(GoBack()));
 	connect(ui.pageNetGame->pNetTeamsWidget, SIGNAL(setEnabledGameStart(bool)),
 		ui.pageNetGame->BtnGo, SLOT(setEnabled(bool)));
+	connect(ui.pageNetGame->pNetTeamsWidget, SIGNAL(setEnabledGameStart(bool)),
+		ui.pageNetGame->BtnStart, SLOT(setEnabled(bool)));
 	connect(ui.pageNetGame->pNetTeamsWidget, SIGNAL(SetupClicked()), this, SLOT(IntermediateSetup()));
+	connect(ui.pageNetGame->pGameCFG, SIGNAL(goToSchemes()), this, SLOT(GoToSchemes()));
+	connect(ui.pageNetGame->pGameCFG, SIGNAL(goToWeapons(const QString &)), this, SLOT(GoToSelectWeaponSet(const QString &)));
 
 	connect(ui.pageRoomsList->BtnBack, SIGNAL(clicked()), this, SLOT(GoBack()));
+	connect(ui.pageRoomsList->BtnAdmin, SIGNAL(clicked()), this, SLOT(GoToAdmin()));
 
 	connect(ui.pageInfo->BtnBack, SIGNAL(clicked()), this, SLOT(GoBack()));
 
 	connect(ui.pageGameStats->BtnBack, SIGNAL(clicked()), this, SLOT(GoBack()));
 
-	connect(ui.pageSinglePlayer->BtnSimpleGamePage,	SIGNAL(clicked()),	this, SLOT(SimpleGame()));
-	connect(ui.pageSinglePlayer->BtnTrainPage,	SIGNAL(clicked()),	this, SLOT(GoToTraining()));
-	connect(ui.pageSinglePlayer->BtnMultiplayer,	SIGNAL(clicked()),	this, SLOT(GoToMultiplayer()));
-	connect(ui.pageSinglePlayer->BtnLoad,	SIGNAL(clicked()),	this, SLOT(GoToSaves()));
-	connect(ui.pageSinglePlayer->BtnDemos,	SIGNAL(clicked()),	this, SLOT(GoToDemos()));
-	connect(ui.pageSinglePlayer->BtnBack,	SIGNAL(clicked()),	this, SLOT(GoBack()));
+	connect(ui.pageSinglePlayer->BtnSimpleGamePage, SIGNAL(clicked()), this, SLOT(SimpleGame()));
+	connect(ui.pageSinglePlayer->BtnTrainPage, SIGNAL(clicked()), this, SLOT(GoToTraining()));
+	connect(ui.pageSinglePlayer->BtnMultiplayer, SIGNAL(clicked()), this, SLOT(GoToMultiplayer()));
+	connect(ui.pageSinglePlayer->BtnLoad, SIGNAL(clicked()), this, SLOT(GoToSaves()));
+	connect(ui.pageSinglePlayer->BtnDemos, SIGNAL(clicked()), this, SLOT(GoToDemos()));
+	connect(ui.pageSinglePlayer->BtnBack, SIGNAL(clicked()), this, SLOT(GoBack()));
 
-	connect(ui.pageTraining->BtnStartTrain,	SIGNAL(clicked()),	this, SLOT(StartTraining()));
-	connect(ui.pageTraining->BtnBack,	SIGNAL(clicked()),	this, SLOT(GoBack()));
+	connect(ui.pageTraining->BtnStartTrain, SIGNAL(clicked()), this, SLOT(StartTraining()));
+	connect(ui.pageTraining->BtnBack, SIGNAL(clicked()), this, SLOT(GoBack()));
 
-	connect(ui.pageSelectWeapon->BtnBack,	SIGNAL(clicked()),	this, SLOT(GoBack()));
-	connect(ui.pageSelectWeapon->BtnDefault,	SIGNAL(clicked()),	ui.pageSelectWeapon->pWeapons, SLOT(setDefault()));
-	connect(ui.pageSelectWeapon->BtnSave,	SIGNAL(clicked()),	ui.pageSelectWeapon->pWeapons, SLOT(save()));
+	connect(ui.pageSelectWeapon->BtnBack, SIGNAL(clicked()), this, SLOT(GoBack()));
+	connect(ui.pageSelectWeapon->BtnDefault, SIGNAL(clicked()), ui.pageSelectWeapon->pWeapons, SLOT(setDefault()));
+	connect(ui.pageSelectWeapon->BtnSave, SIGNAL(clicked()), ui.pageSelectWeapon->pWeapons, SLOT(save()));
 
-	connect(ui.pageSelectWeapon->BtnDelete,	SIGNAL(clicked()),	
+	connect(ui.pageSelectWeapon->BtnDelete, SIGNAL(clicked()),
 		ui.pageSelectWeapon->pWeapons, SLOT(deleteWeaponsName())); // executed first
-	connect(ui.pageSelectWeapon->pWeapons,	SIGNAL(weaponsDeleted()),
+	connect(ui.pageSelectWeapon->pWeapons, SIGNAL(weaponsDeleted()),
 		this, SLOT(UpdateWeapons())); // executed second
-	connect(ui.pageSelectWeapon->pWeapons,	SIGNAL(weaponsDeleted()),
+	connect(ui.pageSelectWeapon->pWeapons, SIGNAL(weaponsDeleted()),
 		this, SLOT(GoBack())); // executed third
 
-	GoToPage(ID_PAGE_MAIN);
+	connect(ui.pageScheme->BtnBack, SIGNAL(clicked()), this, SLOT(GoBack()));
+
+	connect(ui.pageAdmin->BtnBack, SIGNAL(clicked()), this, SLOT(GoBack()));
+
+	connect(ui.pageNetType->BtnBack, SIGNAL(clicked()), this, SLOT(GoBack()));
+	connect(ui.pageNetType->BtnLAN, SIGNAL(clicked()), this, SLOT(GoToNet()));
+	connect(ui.pageNetType->BtnOfficialServer, SIGNAL(clicked()), this, SLOT(NetConnectOfficialServer()));
+
+
+	ammoSchemeModel = new AmmoSchemeModel(this, cfgdir->absolutePath() + "/schemes.ini");
+	ui.pageScheme->setModel(ammoSchemeModel);
+	ui.pageMultiplayer->gameCFG->GameSchemes->setModel(ammoSchemeModel);
+
+    wBackground = NULL;
+    if (config->isFrontendEffects()) {
+       wBackground = new BGWidget(this);
+       wBackground->setFixedSize(this->width(), this->height());
+       wBackground->lower();
+       wBackground->init();
+       wBackground->startAnimation();
+    }
+
+	PagesStack.push(ID_PAGE_MAIN);
+	GoBack();
 }
 
 void HWForm::onFrontendFullscreen(bool value)
@@ -165,36 +211,31 @@ void HWForm::CustomizePalettes()
 
 void HWForm::UpdateWeapons()
 {
-	// FIXME: rewrite this with boost (or TR1/0x)
 	QVector<QComboBox*> combos;
 	combos.push_back(ui.pageOptions->WeaponsName);
 	combos.push_back(ui.pageMultiplayer->gameCFG->WeaponsName);
 	combos.push_back(ui.pageNetGame->pGameCFG->WeaponsName);
 
-	for(QVector<QComboBox*>::iterator it=combos.begin(); it!=combos.end(); ++it) {
+	QStringList names = ui.pageSelectWeapon->pWeapons->getWeaponNames();
+
+	for(QVector<QComboBox*>::iterator it = combos.begin(); it != combos.end(); ++it) {
 		(*it)->clear();
-		(*it)->addItems(ui.pageSelectWeapon->pWeapons->getWeaponNames());
-		int pos=(*it)->findText("Default");
-		if (pos!=-1) {
+
+		for(int i = 0; i < names.size(); ++i)
+			(*it)->addItem(names[i], ui.pageSelectWeapon->pWeapons->getWeaponsString(names[i]));
+		
+		int pos = (*it)->findText("Default");
+		if (pos != -1) {
 			(*it)->setCurrentIndex(pos);
 		}
 	}
 }
 
-void HWForm::NetWeaponNameChanged(const QString& name)
-{
-	QString ammo = ui.pageSelectWeapon->pWeapons->getWeaponsString(
-			ui.pageNetGame->pGameCFG->WeaponsName->currentText()
-			);
-
-	hwnet->onWeaponsNameChanged(name, ammo);
-}
-
 void HWForm::UpdateTeamsLists(const QStringList* editable_teams)
 {
 	QStringList teamslist;
 	if(editable_teams) {
-	  teamslist=*editable_teams;
+	  teamslist =* editable_teams;
 	} else {
 	  teamslist = config->GetTeamsList();
 	}
@@ -231,13 +272,19 @@ void HWForm::GoToSetup()
 
 void HWForm::GoToSelectNewWeapon()
 {
-  ui.pageSelectWeapon->pWeapons->setWeaponsName("", false);
+	ui.pageSelectWeapon->pWeapons->setWeaponsName(tr("new"));
 	GoToPage(ID_PAGE_SELECTWEAPON);
 }
 
 void HWForm::GoToSelectWeapon()
 {
-  ui.pageSelectWeapon->pWeapons->setWeaponsName(ui.pageOptions->WeaponsName->currentText(), true);
+	ui.pageSelectWeapon->pWeapons->setWeaponsName(ui.pageOptions->WeaponsName->currentText());
+	GoToPage(ID_PAGE_SELECTWEAPON);
+}
+
+void HWForm::GoToSelectWeaponSet(const QString & name)
+{
+	ui.pageSelectWeapon->pWeapons->setWeaponsName(name);
 	GoToPage(ID_PAGE_SELECTWEAPON);
 }
 
@@ -272,11 +319,26 @@ void HWForm::GoToNet()
 	GoToPage(ID_PAGE_NET);
 }
 
+void HWForm::GoToNetType()
+{
+	GoToPage(ID_PAGE_NETTYPE);
+}
+
 void HWForm::GoToNetServer()
 {
 	GoToPage(ID_PAGE_NETSERVER);
 }
 
+void HWForm::GoToSchemes()
+{
+	GoToPage(ID_PAGE_SCHEME);
+}
+
+void HWForm::GoToAdmin()
+{
+	GoToPage(ID_PAGE_ADMIN);
+}
+
 void HWForm::OnPageShown(quint8 id, quint8 lastid)
 {
 	if (id == ID_PAGE_MULTIPLAYER || id == ID_PAGE_NETGAME) {
@@ -300,9 +362,16 @@ void HWForm::OnPageShown(quint8 id, quint8 lastid)
 		  if (editedTeam) {
 		    curTeamSelWidget->addTeam(*editedTeam);
 		  }
-		} else if(lastid != ID_PAGE_GAMESTATS && lastid != ID_PAGE_INGAME) {
-		  curTeamSelWidget->resetPlayingTeams(teamsList);
+		} else if(lastid != ID_PAGE_GAMESTATS
+				&& lastid != ID_PAGE_INGAME
+				&& lastid != ID_PAGE_SCHEME
+				&& lastid != ID_PAGE_SELECTWEAPON) {
+			curTeamSelWidget->resetPlayingTeams(teamsList);
 		}
+	} else
+	if (id == ID_PAGE_GAMESTATS)
+	{
+		ui.pageGameStats->renderStats();
 	}
 }
 
@@ -321,12 +390,21 @@ void HWForm::GoBack()
 	ui.Pages->setCurrentIndex(id);
 	OnPageShown(id, curid);
 	
-	if (id == ID_PAGE_ROOMSLIST || id == ID_PAGE_NETSERVER) {
+	if (id == ID_PAGE_CONNECTING)
 		GoBack();
-	}
-	if (id == ID_PAGE_NET) {
-		if(hwnet || pnetserver) NetDisconnect();
-	}
+	if (id == ID_PAGE_NETSERVER)
+		GoBack();
+	if ((!hwnet) && (id == ID_PAGE_ROOMSLIST))
+		GoBack();
+	
+	if ((!hwnet) || (!hwnet->isInRoom()))
+		if (id == ID_PAGE_NETGAME || id == ID_PAGE_NETGAME)
+			GoBack();
+
+	if (curid == ID_PAGE_ROOMSLIST) NetDisconnect();
+
+	if (curid == ID_PAGE_SCHEME)
+		ammoSchemeModel->Save();
 }
 
 void HWForm::btnExitPressed()
@@ -350,21 +428,23 @@ void HWForm::btnExitClicked()
 
 void HWForm::IntermediateSetup()
 {
-  quint8 id=ui.Pages->currentIndex();
-  TeamSelWidget* curTeamSelWidget;
-  if(id == ID_PAGE_MULTIPLAYER) {
-    curTeamSelWidget=ui.pageMultiplayer->teamsSelect;
-  } else {
-    curTeamSelWidget=ui.pageNetGame->pNetTeamsWidget;
-  }
-  QList<HWTeam> teams=curTeamSelWidget->getDontPlayingTeams();
-  QStringList tmnames;
-  for(QList<HWTeam>::iterator it = teams.begin(); it != teams.end(); ++it) {
-    tmnames+=it->TeamName;
-  }
-  UpdateTeamsLists(&tmnames); // FIXME: still need more work if teamname is updated while configuring
+	quint8 id=ui.Pages->currentIndex();
+	TeamSelWidget* curTeamSelWidget;
+	
+	if(id == ID_PAGE_MULTIPLAYER) {
+		curTeamSelWidget = ui.pageMultiplayer->teamsSelect;
+	} else {
+		curTeamSelWidget = ui.pageNetGame->pNetTeamsWidget;
+	}
+	
+	QList<HWTeam> teams = curTeamSelWidget->getDontPlayingTeams();
+	QStringList tmnames;
+	for(QList<HWTeam>::iterator it = teams.begin(); it != teams.end(); ++it) {
+		tmnames += it->TeamName;
+	}
+	UpdateTeamsLists(&tmnames); // FIXME: still need more work if teamname is updated while configuring
 
-  GoToPage(ID_PAGE_SETUP);
+	GoToPage(ID_PAGE_SETUP);
 }
 
 void HWForm::NewTeam()
@@ -382,6 +462,20 @@ void HWForm::EditTeam()
 	GoToPage(ID_PAGE_SETUP_TEAM);
 }
 
+void HWForm::RandomNames()
+{
+	editedTeam->GetFromPage(this);
+	namegen->TeamRandomNames(editedTeam,FALSE);
+	editedTeam->SetToPage(this);
+}
+
+void HWForm::RandomName(const int &i)
+{
+	editedTeam->GetFromPage(this);
+	namegen->TeamRandomName(editedTeam,i);
+	editedTeam->SetToPage(this);
+}
+
 void HWForm::TeamSave()
 {
 	editedTeam->GetFromPage(this);
@@ -401,7 +495,7 @@ void HWForm::TeamDiscard()
 
 void HWForm::SimpleGame()
 {
-	CreateGame(0, 0, cDefaultAmmoStore->mid(10));
+	CreateGame(0, 0, *cDefaultAmmoStore);
 	game->StartQuick();
 }
 
@@ -438,20 +532,32 @@ void HWForm::_NetConnect(const QString & hostName, quint16 port, const QString &
 		hwnet=0;
 	}
 	
-	ui.pageNetGame->pChatWidget->clear();
+	ui.pageRoomsList->chatWidget->clear();
 	
 	hwnet = new HWNewNet(config, ui.pageNetGame->pGameCFG, ui.pageNetGame->pNetTeamsWidget);
 
+	GoToPage(ID_PAGE_CONNECTING);
+
+	connect(hwnet, SIGNAL(showMessage(const QString &)), this, SLOT(ShowErrorMessage(const QString &)), Qt::QueuedConnection);
+
 	connect(hwnet, SIGNAL(AskForRunGame()), this, SLOT(CreateNetGame()));
 	connect(hwnet, SIGNAL(Connected()), this, SLOT(NetConnected()));
 	connect(hwnet, SIGNAL(EnteredGame()), this, SLOT(NetGameEnter()));
+	connect(hwnet, SIGNAL(LeftRoom()), this, SLOT(NetLeftRoom()));
 	connect(hwnet, SIGNAL(AddNetTeam(const HWTeam&)), this, SLOT(AddNetTeam(const HWTeam&)));
+	connect(ui.pageNetGame->BtnBack, SIGNAL(clicked()), hwnet, SLOT(partRoom()));
 
+// rooms list page stuff
 	connect(hwnet, SIGNAL(roomsList(const QStringList&)),
 		ui.pageRoomsList, SLOT(setRoomsList(const QStringList&)));
-	connect(hwnet, SIGNAL(serverMessage(const QString&)),
-		ui.pageRoomsList->serverMessage, SLOT(setText(const QString&)));
+	connect(hwnet, SIGNAL(adminAccess(bool)),
+		ui.pageRoomsList, SLOT(setAdmin(bool)));
+	connect(hwnet, SIGNAL(adminAccess(bool)),
+		ui.pageRoomsList->chatWidget, SLOT(adminAccess(bool)));
 	
+	connect(hwnet, SIGNAL(serverMessage(const QString&)),
+		ui.pageRoomsList->chatWidget, SLOT(onServerMessage(const QString&)));
+
 	connect(ui.pageRoomsList, SIGNAL(askForCreateRoom(const QString &)),
 		hwnet, SLOT(CreateRoom(const QString&)));
 	connect(ui.pageRoomsList, SIGNAL(askForJoinRoom(const QString &)),
@@ -463,22 +569,54 @@ void HWForm::_NetConnect(const QString & hostName, quint16 port, const QString &
 	connect(ui.pageRoomsList, SIGNAL(askForRoomList()),
 		hwnet, SLOT(askRoomsList()));
 
+// net page stuff
 	connect(hwnet, SIGNAL(chatStringFromNet(const QString&)),
 		ui.pageNetGame->pChatWidget, SLOT(onChatString(const QString&)));
 	connect(hwnet, SIGNAL(setReadyStatus(const QString &, bool)),
 		ui.pageNetGame->pChatWidget, SLOT(setReadyStatus(const QString &, bool)));
 	connect(hwnet, SIGNAL(chatStringFromMe(const QString&)),
 		ui.pageNetGame->pChatWidget, SLOT(onChatString(const QString&)));
+	connect(hwnet, SIGNAL(roomMaster(bool)),
+		ui.pageNetGame->pChatWidget, SLOT(adminAccess(bool)));
 	connect(ui.pageNetGame->pChatWidget, SIGNAL(chatLine(const QString&)),
 		hwnet, SLOT(chatLineToNet(const QString&)));
+	connect(ui.pageNetGame->BtnGo, SIGNAL(clicked()), hwnet, SLOT(ToggleReady()));
+	connect(hwnet, SIGNAL(setMyReadyStatus(bool)),
+		ui.pageNetGame, SLOT(setReadyStatus(bool)));
+
+// chat widget actions
 	connect(ui.pageNetGame->pChatWidget, SIGNAL(kick(const QString&)),
 		hwnet, SLOT(kickPlayer(const QString&)));
+	connect(ui.pageNetGame->pChatWidget, SIGNAL(ban(const QString&)),
+		hwnet, SLOT(banPlayer(const QString&)));
+	connect(ui.pageNetGame->pChatWidget, SIGNAL(info(const QString&)),
+		hwnet, SLOT(infoPlayer(const QString&)));
+	connect(ui.pageRoomsList->chatWidget, SIGNAL(kick(const QString&)),
+		hwnet, SLOT(kickPlayer(const QString&)));
+	connect(ui.pageRoomsList->chatWidget, SIGNAL(ban(const QString&)),
+		hwnet, SLOT(banPlayer(const QString&)));
+	connect(ui.pageRoomsList->chatWidget, SIGNAL(info(const QString&)),
+		hwnet, SLOT(infoPlayer(const QString&)));
+
+// chatting
+	connect(ui.pageRoomsList->chatWidget, SIGNAL(chatLine(const QString&)),
+		hwnet, SLOT(chatLineToLobby(const QString&)));
+	connect(hwnet, SIGNAL(chatStringLobby(const QString&)),
+		ui.pageRoomsList->chatWidget, SLOT(onChatString(const QString&)));
+	connect(hwnet, SIGNAL(chatStringFromMeLobby(const QString&)),
+		ui.pageRoomsList->chatWidget, SLOT(onChatString(const QString&)));
+
+// nick list stuff
 	connect(hwnet, SIGNAL(nickAdded(const QString&)),
 		ui.pageNetGame->pChatWidget, SLOT(nickAdded(const QString&)));
 	connect(hwnet, SIGNAL(nickRemoved(const QString&)),
 		ui.pageNetGame->pChatWidget, SLOT(nickRemoved(const QString&)));
-	connect(ui.pageNetGame->BtnGo, SIGNAL(clicked()), hwnet, SLOT(ToggleReady()));
+	connect(hwnet, SIGNAL(nickAddedLobby(const QString&)),
+		ui.pageRoomsList->chatWidget, SLOT(nickAdded(const QString&)));
+	connect(hwnet, SIGNAL(nickRemovedLobby(const QString&)),
+		ui.pageRoomsList->chatWidget, SLOT(nickRemoved(const QString&)));
 
+// teams selecting stuff
 	connect(ui.pageNetGame->pNetTeamsWidget, SIGNAL(hhogsNumChanged(const HWTeam&)),
 		hwnet, SLOT(onHedgehogsNumChanged(const HWTeam&)));
 	connect(ui.pageNetGame->pNetTeamsWidget, SIGNAL(teamColorChanged(const HWTeam&)),
@@ -486,29 +624,18 @@ void HWForm::_NetConnect(const QString & hostName, quint16 port, const QString &
 	connect(ui.pageNetGame->pNetTeamsWidget, SIGNAL(teamWillPlay(HWTeam)), hwnet, SLOT(AddTeam(HWTeam)));
 	connect(ui.pageNetGame->pNetTeamsWidget, SIGNAL(acceptRequested(HWTeam)), hwnet, SLOT(AddTeam(HWTeam)));
 	connect(ui.pageNetGame->pNetTeamsWidget, SIGNAL(teamNotPlaying(const HWTeam&)), hwnet, SLOT(RemoveTeam(const HWTeam&)));
-
-	connect(ui.pageNetGame->pGameCFG, SIGNAL(seedChanged(const QString &)), hwnet, SLOT(onSeedChanged(const QString &)));
-	connect(ui.pageNetGame->pGameCFG, SIGNAL(mapChanged(const QString &)), hwnet, SLOT(onMapChanged(const QString &)));
-	connect(ui.pageNetGame->pGameCFG, SIGNAL(themeChanged(const QString &)), hwnet, SLOT(onThemeChanged(const QString &)));
-	connect(ui.pageNetGame->pGameCFG, SIGNAL(initHealthChanged(int)), hwnet, SLOT(onInitHealthChanged(int)));
-	connect(ui.pageNetGame->pGameCFG, SIGNAL(turnTimeChanged(int)), hwnet, SLOT(onTurnTimeChanged(int)));
-	connect(ui.pageNetGame->pGameCFG, SIGNAL(fortsModeChanged(bool)), hwnet, SLOT(onFortsModeChanged(bool)));
-	connect(ui.pageNetGame->pGameCFG, SIGNAL(teamsDivideChanged(bool)), hwnet, SLOT(onTeamsDivideChanged(bool)));
-
-	connect(hwnet, SIGNAL(Disconnected()), this, SLOT(ForcedDisconnect()));
-	connect(hwnet, SIGNAL(seedChanged(const QString &)), ui.pageNetGame->pGameCFG, SLOT(setSeed(const QString &)));
-	connect(hwnet, SIGNAL(mapChanged(const QString &)), ui.pageNetGame->pGameCFG, SLOT(setMap(const QString &)));
-	connect(hwnet, SIGNAL(themeChanged(const QString &)), ui.pageNetGame->pGameCFG, SLOT(setTheme(const QString &)));
-	connect(hwnet, SIGNAL(initHealthChanged(int)), ui.pageNetGame->pGameCFG, SLOT(setInitHealth(int)));
-	connect(hwnet, SIGNAL(turnTimeChanged(int)), ui.pageNetGame->pGameCFG, SLOT(setTurnTime(int)));
-	connect(hwnet, SIGNAL(fortsModeChanged(bool)), ui.pageNetGame->pGameCFG, SLOT(setFortsMode(bool)));
-	connect(hwnet, SIGNAL(teamsDivideChanged(bool)), ui.pageNetGame->pGameCFG, SLOT(setTeamsDivide(bool)));
 	connect(hwnet, SIGNAL(hhnumChanged(const HWTeam&)),
 		ui.pageNetGame->pNetTeamsWidget, SLOT(changeHHNum(const HWTeam&)));
 	connect(hwnet, SIGNAL(teamColorChanged(const HWTeam&)),
 		ui.pageNetGame->pNetTeamsWidget, SLOT(changeTeamColor(const HWTeam&)));
-	connect(hwnet, SIGNAL(ammoChanged(const QString&, const QString&)), ui.pageNetGame->pGameCFG, SLOT(setNetAmmo(const QString&, const QString&)));
 
+// admin stuff
+	connect(hwnet, SIGNAL(serverMessage(const QString&)), ui.pageAdmin, SLOT(serverMessage(const QString &)));
+	connect(ui.pageAdmin, SIGNAL(setServerMessage(const QString&)), hwnet, SLOT(newServerMessage(const QString &)));
+
+// disconnect
+	connect(hwnet, SIGNAL(Disconnected()), this, SLOT(ForcedDisconnect()), Qt::QueuedConnection);
+	
 	hwnet->Connect(hostName, port, nick);
 }
 
@@ -536,7 +663,7 @@ void HWForm::NetStartServer()
 	if(!pnetserver->StartServer(ui.pageNetServer->sbPort->value()))
 	{
 		QMessageBox::critical(0, tr("Error"),
-					tr("Unable to start the server"));
+				tr("Unable to start the server"));
 		delete pnetserver;
 		pnetserver = 0;
 		return;
@@ -556,22 +683,23 @@ void HWForm::AsyncNetServerStart()
 
 void HWForm::NetDisconnect()
 {
-  if(hwnet) {
-    hwnet->Disconnect();
-    delete hwnet;
-    hwnet = 0;
-  }
-  if(pnetserver) {
-    if (pRegisterServer)
-    {
-      pRegisterServer->unregister();
-      pRegisterServer = 0;
-    }
+	//qDebug("NetDisconnect");
+	if(hwnet) {
+		hwnet->Disconnect();
+		delete hwnet;
+		hwnet = 0;
+	}
+	if(pnetserver) {
+		if (pRegisterServer)
+		{
+			pRegisterServer->unregister();
+			pRegisterServer = 0;
+		}
 
-    pnetserver->StopServer();
-    delete pnetserver;
-    pnetserver = 0;
-  }
+		pnetserver->StopServer();
+		delete pnetserver;
+		pnetserver = 0;
+	}
 }
 
 void HWForm::ForcedDisconnect()
@@ -582,8 +710,9 @@ void HWForm::ForcedDisconnect()
 		hwnet = 0;
 		QMessageBox::warning(this, QMessageBox::tr("Network"),
 				QMessageBox::tr("Connection to server is lost"));
+	
 	}
-	GoBack();
+	if (ui.Pages->currentIndex() != ID_PAGE_NET) GoBack();
 }
 
 void HWForm::NetConnected()
@@ -593,6 +722,7 @@ void HWForm::NetConnected()
 
 void HWForm::NetGameEnter()
 {
+	ui.pageNetGame->pChatWidget->clear();
 	GoToPage(ID_PAGE_NETGAME);
 }
 
@@ -603,7 +733,10 @@ void HWForm::AddNetTeam(const HWTeam& team)
 
 void HWForm::StartMPGame()
 {
-	QString ammo = ui.pageSelectWeapon->pWeapons->getWeaponsString(ui.pageMultiplayer->gameCFG->WeaponsName->currentText());
+	QString ammo;
+	ammo = ui.pageMultiplayer->gameCFG->WeaponsName->itemData(
+		ui.pageMultiplayer->gameCFG->WeaponsName->currentIndex()
+		).toString();
 
 	CreateGame(ui.pageMultiplayer->gameCFG, ui.pageMultiplayer->teamsSelect, ammo);
 
@@ -615,8 +748,9 @@ void HWForm::GameStateChanged(GameState gameState)
 	switch(gameState) {
 		case gsStarted: {
 			Music(false);
+			if (wBackground) wBackground->stopAnimation();	
 			GoToPage(ID_PAGE_INGAME);
-			ui.pageGameStats->labelGameStats->setText("");
+			ui.pageGameStats->clear();
 			if (pRegisterServer)
 			{
 				pRegisterServer->unregister();
@@ -627,6 +761,7 @@ void HWForm::GameStateChanged(GameState gameState)
 		case gsFinished: {
 			GoBack();
 			Music(ui.pageOptions->CBEnableMusic->isChecked());
+			if (wBackground) wBackground->startAnimation();	
 			GoToPage(ID_PAGE_GAMESTATS);
 			if (hwnet) hwnet->gameFinished();
 			break;
@@ -636,52 +771,18 @@ void HWForm::GameStateChanged(GameState gameState)
 			if (id == ID_PAGE_INGAME) {
 				GoBack();
 				Music(ui.pageOptions->CBEnableMusic->isChecked());
+				if (wBackground) wBackground->startAnimation();	
 				if (hwnet) hwnet->gameFinished();
 			}
 		};
 	}
 }
 
-void HWForm::AddStatText(const QString & msg)
-{
-	ui.pageGameStats->labelGameStats->setText(
-		ui.pageGameStats->labelGameStats->text() + msg);
-}
-
-void HWForm::GameStats(char type, const QString & info)
-{
-	switch(type) {
-		case 'r' : {
-			AddStatText(QString("<h1 align=\"center\">%1</h1>").arg(info));
-			break;
-		}
-		case 'D' : {
-			int i = info.indexOf(' ');
-			QString message = QLabel::tr("<p>The best shot award was won by <b>%1</b> with <b>%2</b> pts.</p>")
-					.arg(info.mid(i + 1), info.left(i));
-			AddStatText(message);
-			break;
-		}
-		case 'k' : {
-			int i = info.indexOf(' ');
-			QString message = QLabel::tr("<p>The best shot award was won by <b>%1</b> with <b>%2</b> kills.</p>")
-					.arg(info.mid(i + 1), info.left(i));
-			AddStatText(message);
-			break;
-		}
-		case 'K' : {
-			QString message = QLabel::tr("<p>A total of <b>%1</b> Hedgehog(s) were killed during this round.</p>").arg(info);
-			AddStatText(message);
-			break;
-		}
-	}
-}
-
 void HWForm::CreateGame(GameCFGWidget * gamecfg, TeamSelWidget* pTeamSelWidget, QString ammo)
 {
 	game = new HWGame(config, gamecfg, ammo, pTeamSelWidget);
 	connect(game, SIGNAL(GameStateChanged(GameState)), this, SLOT(GameStateChanged(GameState)));
-	connect(game, SIGNAL(GameStats(char, const QString &)), this, SLOT(GameStats(char, const QString &)));
+	connect(game, SIGNAL(GameStats(char, const QString &)), ui.pageGameStats, SLOT(GameStats(char, const QString &)));
 	connect(game, SIGNAL(ErrorMessage(const QString &)), this, SLOT(ShowErrorMessage(const QString &)), Qt::QueuedConnection);
 	connect(game, SIGNAL(HaveRecord(bool, const QByteArray &)), this, SLOT(GetRecord(bool, const QByteArray &)));
 }
@@ -697,17 +798,22 @@ void HWForm::GetRecord(bool isDemo, const QByteArray & record)
 {
 	QString filename;
 	QByteArray demo = record;
+	QString recordFileName =
+			config->appendDateTimeToRecordName() ?
+				QDateTime::currentDateTime().toString("yyyy-MM-dd_hh-mm") :
+				"LastRound";
+
 	if (isDemo)
 	{
 		demo.replace(QByteArray("\x02TL"), QByteArray("\x02TD"));
 		demo.replace(QByteArray("\x02TN"), QByteArray("\x02TD"));
 		demo.replace(QByteArray("\x02TS"), QByteArray("\x02TD"));
-		filename = cfgdir->absolutePath() + "/Demos/LastRound.hwd_" + *cProtoVer;
+		filename = cfgdir->absolutePath() + "/Demos/" + recordFileName + ".hwd_" + *cProtoVer;
 	} else
 	{
 		demo.replace(QByteArray("\x02TL"), QByteArray("\x02TS"));
 		demo.replace(QByteArray("\x02TN"), QByteArray("\x02TS"));
-		filename = cfgdir->absolutePath() + "/Saves/LastRound.hws_" + *cProtoVer;
+		filename = cfgdir->absolutePath() + "/Saves/" + recordFileName + ".hws_" + *cProtoVer;
 	}
 
 
@@ -731,12 +837,10 @@ void HWForm::StartTraining()
 void HWForm::CreateNetGame()
 {
 	QString ammo;
-	if (hwnet->isRoomChief()) {
-		ammo = ui.pageSelectWeapon->pWeapons->getWeaponsString(ui.pageNetGame->pGameCFG->WeaponsName->currentText());
-	} else {
-		ammo = ui.pageNetGame->pGameCFG->getNetAmmo();
-	}
-	
+	ammo = ui.pageNetGame->pGameCFG->WeaponsName->itemData(
+			ui.pageNetGame->pGameCFG->WeaponsName->currentIndex()
+			).toString();
+
 	CreateGame(ui.pageNetGame->pGameCFG, ui.pageNetGame->pNetTeamsWidget, ammo);
 
 	connect(game, SIGNAL(SendNet(const QByteArray &)), hwnet, SLOT(SendNet(const QByteArray &)));
@@ -763,19 +867,63 @@ void HWForm::Music(bool checked)
 
 void HWForm::NetGameMaster()
 {
-	ui.pageNetGame->BtnMaster->setVisible(true);
+	ui.pageNetGame->setMasterMode(true);
 	ui.pageNetGame->restrictJoins->setChecked(false);
 	ui.pageNetGame->restrictTeamAdds->setChecked(false);
+	ui.pageNetGame->pGameCFG->GameSchemes->setModel(ammoSchemeModel);
 	
 	if (hwnet)
 	{
-		connect(ui.pageNetGame->startGame, SIGNAL(triggered()), hwnet, SLOT(startGame()));
+		// disconnect connections first to ensure their inexistance and not to connect twice
+		ui.pageNetGame->BtnStart->disconnect(hwnet);
+		ui.pageNetGame->restrictJoins->disconnect(hwnet);
+		ui.pageNetGame->restrictTeamAdds->disconnect(hwnet);
+		connect(ui.pageNetGame->BtnStart, SIGNAL(clicked()), hwnet, SLOT(startGame()));
 		connect(ui.pageNetGame->restrictJoins, SIGNAL(triggered()), hwnet, SLOT(toggleRestrictJoins()));
 		connect(ui.pageNetGame->restrictTeamAdds, SIGNAL(triggered()), hwnet, SLOT(toggleRestrictTeamAdds()));
+		connect(ui.pageNetGame->pGameCFG->GameSchemes->model(),
+				SIGNAL(dataChanged(const QModelIndex &, const QModelIndex &)),
+				ui.pageNetGame->pGameCFG,
+				SLOT(resendSchemeData())
+				);
 	}
 }
 
 void HWForm::NetGameSlave()
 {
-	ui.pageNetGame->BtnMaster->setVisible(false);
+	if (hwnet)
+	{
+		NetAmmoSchemeModel * netAmmo = new NetAmmoSchemeModel(hwnet);
+		connect(hwnet, SIGNAL(netSchemeConfig(QStringList &)), netAmmo, SLOT(setNetSchemeConfig(QStringList &)));
+		ui.pageNetGame->pGameCFG->GameSchemes->setModel(netAmmo);
+
+		ui.pageNetGame->pGameCFG->GameSchemes->view()->disconnect(hwnet);
+		connect(hwnet, SIGNAL(netSchemeConfig(QStringList &)),
+				this, SLOT(selectFirstNetScheme()));
+	}
+
+	ui.pageNetGame->setMasterMode(false);
+}
+
+void HWForm::selectFirstNetScheme()
+{
+	ui.pageNetGame->pGameCFG->GameSchemes->setCurrentIndex(0);
+}
+
+void HWForm::NetLeftRoom()
+{
+	if (ui.Pages->currentIndex() == ID_PAGE_NETGAME)
+		GoBack();
+	else
+		qWarning("Left room while not in room");
+}
+
+void HWForm::resizeEvent(QResizeEvent * event)
+{
+	int w = event->size().width();
+	int h = event->size().height();
+    if (wBackground) {
+        wBackground->setFixedSize(w, h);
+        wBackground->move(0, 0);
+    }
 }
