@@ -27,13 +27,18 @@
 #include "game.h"
 #include "ui_hwform.h"
 #include "SDLs.h"
+#include "bgwidget.h"
 
 class HWGame;
 class HWTeam;
+class HWNamegen;
 class HWNewNet;
 class GameUIConfig;
 class HWNetRegisterServer;
 class QCloseEvent;
+class AmmoSchemeModel;
+
+extern bool frontendEffects;
 
 class HWForm : public QMainWindow
 {
@@ -51,11 +56,15 @@ private slots:
 	void GoToSaves();
 	void GoToDemos();
 	void GoToNet();
+	void GoToNetType();
 	void GoToInfo();
 	void GoToTraining();
 	void GoToSelectWeapon();
+	void GoToSelectWeaponSet(const QString & name);
 	void GoToSelectNewWeapon();
 	void GoToNetServer();
+	void GoToSchemes();
+	void GoToAdmin();
 	void GoToPage(quint8 id);
 	void GoBack();
 	void btnExitPressed();
@@ -63,6 +72,8 @@ private slots:
 	void IntermediateSetup();
 	void NewTeam();
 	void EditTeam();
+	void RandomNames();
+	void RandomName(const int &i);
 	void TeamSave();
 	void TeamDiscard();
 	void SimpleGame();
@@ -78,18 +89,18 @@ private slots:
 	void AddNetTeam(const HWTeam& team);
 	void StartMPGame();
 	void GameStateChanged(GameState gameState);
-	void GameStats(char type, const QString & info);
 	void ForcedDisconnect();
 	void ShowErrorMessage(const QString &);
 	void GetRecord(bool isDemo, const QByteArray & record);
 	void CreateNetGame();
 	void UpdateWeapons();
-	void NetWeaponNameChanged(const QString& name);
 	void onFrontendFullscreen(bool value);
 	void Music(bool checked);
 	void NetGameMaster();
 	void NetGameSlave();
 	void AsyncNetServerStart();
+	void NetLeftRoom();
+	void selectFirstNetScheme();
 
 private:
 	void _NetConnect(const QString & hostName, quint16 port, const QString & nick);
@@ -97,6 +108,7 @@ private:
 	void CreateGame(GameCFGWidget * gamecfg, TeamSelWidget* pTeamSelWidget, QString ammo);
 	void closeEvent(QCloseEvent *event);
 	void CustomizePalettes();
+	void resizeEvent(QResizeEvent * event);
 	
 	enum PageIDs {
 		ID_PAGE_SETUP_TEAM      =  0,
@@ -113,18 +125,24 @@ private:
 		ID_PAGE_SELECTWEAPON    = 11,
 		ID_PAGE_NETSERVER       = 12,
 		ID_PAGE_INGAME          = 13,
-		ID_PAGE_ROOMSLIST       = 14
+		ID_PAGE_ROOMSLIST       = 14,
+		ID_PAGE_CONNECTING      = 15,
+		ID_PAGE_SCHEME          = 16,
+		ID_PAGE_ADMIN           = 17,
+		ID_PAGE_NETTYPE         = 18
 		};
 	HWGame * game;
+	HWNetServer* pnetserver;
+	HWNetRegisterServer* pRegisterServer;
 	HWTeam * editedTeam;
 	HWNewNet * hwnet;
 	GameUIConfig * config;
+	HWNamegen * namegen;
+	AmmoSchemeModel * ammoSchemeModel;
 	QStack<quint8> PagesStack;
-	HWNetServer* pnetserver;
-	HWNetRegisterServer* pRegisterServer;
 	QTime eggTimer;
 	SDLInteraction sdli;
-	void AddStatText(const QString & msg);
+	BGWidget * wBackground;
 	void OnPageShown(quint8 id, quint8 lastid=0);
 };
 
