@@ -34,7 +34,7 @@ class HWMap : public TCPBase
  public:
   HWMap();
   virtual ~HWMap();
-  void getImage(std::string seed);
+  void getImage(std::string seed, int templateFilter);
 
  protected:
   virtual QStringList setArguments();
@@ -43,9 +43,11 @@ class HWMap : public TCPBase
 
  signals:
   void ImageReceived(const QImage newImage);
+  void HHLimitReceived(int hhLimit);
 
  private:
   std::string m_seed;
+  int templateFilter;
 
  private slots:
 };
