@@ -17,15 +17,16 @@
  */
  
 #include "itemNum.h"
+#include "hwform.h"
 
 #include <QMouseEvent>
 #include <QPainter>
 
 ItemNum::ItemNum(const QImage& im, QWidget * parent, unsigned char min, unsigned char max) :
-  m_im(im), QFrame(parent), nonInteractive(false), minItems(min), maxItems(max),
-  numItems(min+2 >= max ? min : min+2),
-  infinityState(false)
+  QFrame(parent), m_im(im), infinityState(false), nonInteractive(false), minItems(min), maxItems(max),
+  numItems(min+2 >= max ? min : min+2)
 {
+    if(frontendEffects) setAttribute(Qt::WA_PaintOnScreen, true);
 }
 
 ItemNum::~ItemNum()
@@ -37,16 +38,20 @@ void ItemNum::mousePressEvent ( QMouseEvent * event )
   if(nonInteractive) return;
   if(event->button()==Qt::LeftButton) {
     event->accept();
-    if(infinityState && numItems==maxItems) {
-      incItems();
-    }
-    if(numItems < maxItems) {
+    if((infinityState && numItems <= maxItems) || (!infinityState && numItems < maxItems)) {
       incItems();
+    } else {
+      numItems = minItems+1;
+      // appears there's an emit in there
+      decItems();
     }
   } else if (event->button()==Qt::RightButton) {
     event->accept();
     if(numItems > minItems) {
       decItems();
+    } else {
+      numItems = maxItems+(infinityState?0:-1);
+      incItems();
     }
   } else {
     event->ignore();
