@@ -21,6 +21,8 @@
 #include <QLocale>
 #include <QMessageBox>
 #include <QPlastiqueStyle>
+#include <QRegExp>
+#include <QMap>
 
 #include "hwform.h"
 #include "hwconsts.h"
@@ -44,6 +46,31 @@ int main(int argc, char *argv[])
 {
 	QApplication app(argc, argv);
 
+    QStringList arguments = app.arguments();
+    QMap<QString, QString> parsedArgs;
+    {
+        QList<QString>::iterator i = arguments.begin();
+        while(i != arguments.end()) {
+            QString arg = *i;
+
+            QRegExp opt("--(\\S+)=(.+)");
+            if(opt.exactMatch(arg)) {
+                parsedArgs[opt.cap(1)] = opt.cap(2);
+                i = arguments.erase(i);
+            } else {
+              ++i;
+            }
+        }
+    }
+
+    if(parsedArgs.contains("data-dir")) {
+        QFileInfo f(parsedArgs["data-dir"]);
+        if(!f.exists()) {
+            qWarning() << "WARNING: Cannot open DATA_PATH=" << f.absoluteFilePath();
+        }
+        *cDataDir = f.absoluteFilePath();
+    }
+
 	app.setStyle(new QPlastiqueStyle);
 	
 	QDateTime now = QDateTime::currentDateTime();
@@ -62,175 +89,139 @@ int main(int argc, char *argv[])
 				"background-color: #870c8f;"
 				"}"
 
-			"QPushButton{"
+			"* {"
+				"color: #ffcc00;"
+			"}"
+
+			"QLineEdit, QListWidget, QTableView, QTextBrowser, QSpinBox, QComboBox, "
+            "QComboBox QAbstractItemView, QMenu::item {"
+				"background-color: #0d0544;"
+			"}"
+
+			"QPushButton, QListWidget, QTableView, QLineEdit, QHeaderView, "
+			"QTextBrowser, QSpinBox, QToolBox, QComboBox, "
+            "QComboBox QAbstractItemView, IconedGroupBox, "
+			".QGroupBox, GameCFGWidget, TeamSelWidget, SelWeaponWidget, "
+            "QTabWidget::pane, QTabBar::tab {"
 				"border: solid;"
 				"border-width: 3px;"
-				"border-radius: 10px;"
 				"border-color: #ffcc00;"
+			"}"
+
+			"QPushButton:hover, QLineEdit:hover, QListWidget:hover, "
+			"QSpinBox:hover, QToolBox:hover, QComboBox:hover {"
+			    "border-color: yellow;"
+			"}"
+
+			"QLineEdit, QListWidget,QTableView, QTextBrowser, "
+			"QSpinBox, QToolBox { "
+				"border-radius: 12px;"
+			"}"
+
+			"QLineEdit, QLabel, QHeaderView, QListWidget, QTableView, "
+			"QSpinBox, QToolBox::tab, QComboBox, QComboBox QAbstractItemView, "
+			"IconedGroupBox, .QGroupBox, GameCFGWidget, TeamSelWidget, "
+            "SelWeaponWidget, QCheckBox, QRadioButton {"
+				"font: bold 14px;"
+			"}"
+
+			".QGroupBox,GameCFGWidget,TeamSelWidget,SelWeaponWidget {"
+				"background-image: url(\":/res/panelbg.png\");"
+				"background-position: bottom center;"
+				"background-repeat: repeat-x;"
+				"border-radius: 16px;"
+				"background-color: #040200;"
+				"padding: 6px;"
+			"}"
+/*  Experimenting with PaintOnScreen and border-radius on IconedGroupBox children didn't work out well
+			"IconedGroupBox QComboBox, IconedGroupBox QPushButton, IconedGroupBox QLineEdit, "
+            "IconedGroupBox QSpinBox {"
+				"border-radius: 0;"
+			"}"
+			"IconedGroupBox, IconedGroupBox *, QTabWidget::pane, QTabBar::tab:selected, QToolBox::tab QWidget{" */
+			"IconedGroupBox, QTabWidget::pane, QTabBar::tab:selected, QToolBox::tab QWidget{"
+				"background-color: #130f2c;"
+			"}"
+
+
+			"QPushButton {"
+				"border-radius: 10px;"
 				"background-origin: margin;"
 				"background-position: top left;"
 				"background-color: #00351d;"
-				"color: #ffcc00;"
-				"}"
-				"QPushButton:hover{"
-				"border-color: yellow;"
-				"}"
-				"QPushButton:pressed{"
-				"border-color: white;"
-				"}"
+			"}"
 
-			"QLineEdit{"
-				"border: solid;"
-				"border-width: 3px;"
-				"border-radius: 12px;"
-				"border-color: #ffcc00;"
-				"background-color: #0d0544;"
-				"color: #ffcc00;"
-				"font: bold 14px;"
-				"}"
-			"QLineEdit:hover{"
-				"border-color: yellow;"
-				"}"
+			"QPushButton:pressed{"
+			    "border-color: white;"
+			"}"
 
-			"QLabel{"
-				"color: #ffcc00;"
-				"font: bold 14px;"
-				"}"
+			"QHeaderView {"
+				"border-radius: 0;"
+				"border-width: 0;"
+				"border-bottom-width: 3px;"
+				"background-color: #00351d;"
+			"}"
+			"QTableView {"
+				"alternate-background-color: #2f213a;"
+			"}"
 
-			"QListWidget,QTableView{"
-				"border: solid;"
-				"border-width: 3px;"
-				"border-radius: 12px;"
-				"border-color: #ffcc00;"
-				"background-color: #0d0544;"
-//				"alternate-background-color: #2f213a;" //what's it?
-				"color: #ffcc00;"
-				"font: bold 14px;"
-				"}"
-			"QListWidget:hover{"
-				"border-color: yellow;"
-				"}"
-
-			"QTextBrowser{"
-				"border: solid;"
-				"border-width: 3px;"
-				"border-radius: 12px;"
-				"border-color: #ffcc00;"
-				"background-color: #0d0544;"
-				"color: #ffcc00;"
-				"font: bold 14px;"
-				"}"
-
-			"QSpinBox{"
-				"border: solid;"
-				"border-width: 3px;"
-				"border-radius: 12px;"
-				"border-color: #ffcc00;"
-				"background-color: #0d0544;"
-				"color: #ffcc00;"
-				"font: bold 14px;"
-				"}"
-			"QSpinBox:hover{"
-				"border-color: yellow;"
-				"}"
+            "QTabBar::tab {"
+                 "border-bottom-width: 0;"
+                 "border-radius: 0;"
+                 "border-top-left-radius: 6px;"
+                 "border-top-right-radius: 6px;"
+                 "padding: 3px;"
+            "}"
+            "QTabBar::tab:!selected {"
+                 "color: #0d0544;"
+                 "background-color: #ffcc00;"
+            "}"
 			"QSpinBox::up-button{"
 				"background: transparent;"
 				"width: 16px;"
 				"height: 10px;"
-				"}"
-			"QSpinBox::up-arrow{"
+			"}"
+
+			"QSpinBox::up-arrow {"
 				"image: url(\":/res/spin_up.png\");"
-				//"width: 5px;"
-				//"height: 5px;"
-				"}"
-			"QSpinBox::down-button{"
+			"}"
+
+			"QSpinBox::down-arrow {"
+				"image: url(\":/res/spin_down.png\");"
+			"}"
+
+			"QSpinBox::down-button {"
 				"background: transparent;"
 				"width: 16px;"
 				"height: 10px;"
-				"}"
-			"QSpinBox::down-arrow{"
-				"image: url(\":/res/spin_down.png\");"
-				"}"
-
-			"QToolBox{"
-				"border: solid;"
-				"border-width: 3px;"
-				"border-radius: 12px;"
-				"border-color: #ffcc00;"
-				//"background-color: #0d0544;"
 			"}"
-			"QToolBox::tab{"
-				"color: #ffcc00;"
-				"font: bold 14px;"
-				"}"
-			"QToolBox:hover{"
-				"border-color: yellow;"
-				"}"
 
-			"QComboBox{"
-				"border: solid;"
-				"border-width: 3px;"
+			"QComboBox {"
 				"border-radius: 15px;"
-				"border-color: #ffcc00;"
-				"background-color: #0d0544;"
-				"color: #ffcc00;"
-				"font: bold 14px;"
 				"padding: 3px;"
-//				"text-align: center;"
-				"}"
-			"QComboBox:hover{"
-				"border-color: yellow;"
-				"}"
+			"}"
 			"QComboBox:pressed{"
 				"border-color: white;"
-				"}"
+			"}"
 			"QComboBox::drop-down{"
 				"border: transparent;"
 				"width: 25px;"
-				"}"
+			"}"
 			"QComboBox::down-arrow {"
 				"image: url(\":/res/dropdown.png\");"
-				"}"
-			"QComboBox QAbstractItemView{"
-				"border: solid transparent;"
-				"border-width: 3px;"
-				//"border-radius: 12px;" -- bad corners look
-				"border-color: #ffcc00;"
-				"background-color: #0d0544;"
-				"color: #ffcc00;"
-				"font: bold 14px;"
-				"}"
+			"}"
 			
-			"VertScrArea{"
+			"VertScrArea {"
 				"background-image: url(\":/res/panelbg.png\");"
 				"background-position: bottom center;"
 				"background-repeat: repeat-x;"
-				"}"
+			"}"
 			
-			"IconedGroupBox{"
-				"border: solid;"
-				"border-width: 3px;"
+			"IconedGroupBox {"
 				"border-radius: 16px;"
-				"border-color: #ffcc00;"
-				"background-color: #130f2c;"
-				"color: #ffcc00;"
-				"font: bold 14px;"
 				"padding: 2px;"
-				"}"
-			".QGroupBox,GameCFGWidget,TeamSelWidget,SelWeaponWidget{"
-				"background-image: url(\":/res/panelbg.png\");"
-				"background-position: bottom center;"
-				"background-repeat: repeat-x;"
-				"border: solid;"
-				"border-width: 3px;"
-				"border-radius: 16px;"
-				"border-color: #ffcc00;"
-				"background-color: #040200;"
-				"padding: 6px;"
-				"color: #ffcc00;"
-				"font: bold 14px;"
-				//"margin-top: 24px;"
-				"}"
+			"}"
+
 			".QGroupBox::title{"
 				"subcontrol-origin: margin;"
 				"subcontrol-position: top left;"
@@ -239,10 +230,6 @@ int main(int argc, char *argv[])
 				"text-align: left;"
 				"}"
 
-			"QCheckBox{"
-				"color: #ffcc00;"
-				"font: bold 14px;"
-				"}"
 			"QCheckBox::indicator:checked{"
 				"image: url(\":/res/checked.png\");"
 				"}"
@@ -250,64 +237,69 @@ int main(int argc, char *argv[])
 				"image: url(\":/res/unchecked.png\");"
 				"}"
 			
-			"QRadioButton{"
-				"color: #ffcc00;"
-				"font: bold 14px;"
-				"}"
-			
 			".QWidget{"
 				"background: transparent;"
 				"}"
-			".QTabWidget{"
-				//"background: transparent;"
-				"background-color: #130f2c;"
-				"}"
+
+			"QTabWidget::pane {"
+                "border-top-width: 2px;"
+			"}"
 
 			"QMenu{"
 				"background-color: #ffcc00;"
 				"margin: 3px;"
-				"}"
-			"QMenu::item{"
+			"}"
+			"QMenu::item {"
 				"background-color: #0d0544;"
 				"border: 1px solid transparent;"
 				"font: bold;"
-				"color: #ffcc00;"
 				"padding: 2px 25px 2px 20px;"
-				"}"
-			"QMenu::item:selected{"
-				"border-color: yellow;"
+			"}"
+			"QMenu::item:selected {"
 				"background-color: #2d2564;"
-				"}"
-			"QMenu::indicator{"
+			"}"
+			"QMenu::indicator {"
 				"width: 16px;"
 				"height: 16px;"
-				"}"
+			"}"
 			"QMenu::indicator:non-exclusive:checked{"
 				"image: url(\":/res/checked.png\");"
-				"}"
+			"}"
 			"QMenu::indicator:non-exclusive:unchecked{"
 				"image: url(\":/res/unchecked.png\");"
-				"}"
+			"}"
 
 			"QToolTip{"
 				"background-color: #0d0544;"
-				"}"
+			"}"
 			
 			":disabled{"
 				"color: #a0a0a0;"
-				"}"
+			"}"
+            "SquareLabel, ItemNum {"
+				"background-color: #000000;"
+			"}"
 			)
 		);
 
 	bindir->cd("bin"); // workaround over NSIS installer
 
 	cfgdir->setPath(cfgdir->homePath());
+#ifdef __APPLE__
+	if (checkForDir(cfgdir->absolutePath() + "/Library/Application Support/Hedgewars"))
+	{
+		checkForDir(cfgdir->absolutePath() + "/Library/Application Support/Hedgewars/Demos");
+		checkForDir(cfgdir->absolutePath() + "/Library/Application Support/Hedgewars/Saves");
+	}
+	cfgdir->cd("Library/Application Support/Hedgewars");
+#else
 	if (checkForDir(cfgdir->absolutePath() + "/.hedgewars"))
 	{
 		checkForDir(cfgdir->absolutePath() + "/.hedgewars/Demos");
 		checkForDir(cfgdir->absolutePath() + "/.hedgewars/Saves");
 	}
 	cfgdir->cd(".hedgewars");
+#endif
 
 	datadir->cd(bindir->absolutePath());
 	datadir->cd(*cDataDir);
