@@ -39,10 +39,15 @@ HWMapContainer::HWMapContainer(QWidget * parent) :
 	mainLayout(this),
 	pMap(0)
 {
+	hhSmall.load(":/res/hh_small.png");
+	hhLimit = 18;
+    templateFilter = 0;
+	
 	mainLayout.setContentsMargins(QApplication::style()->pixelMetric(QStyle::PM_LayoutLeftMargin),
 		1,
 		QApplication::style()->pixelMetric(QStyle::PM_LayoutRightMargin),
 		QApplication::style()->pixelMetric(QStyle::PM_LayoutBottomMargin));
+	
 	imageButt = new QPushButton(this);
 	imageButt->setObjectName("imageButt");
 	imageButt->setFixedSize(256 + 6, 128 + 6);
@@ -61,11 +66,20 @@ HWMapContainer::HWMapContainer(QWidget * parent) :
 				QString("%1/Maps/%2/map.cfg")
 				.arg(datadir->absolutePath())
 				.arg(map));
+
 		if (mapCfgFile.open(QFile::ReadOnly)) {
 			QString theme;
+			quint32 limit = 0;
+			QList<QVariant> mapInfo;
 			QTextStream input(&mapCfgFile);
 			input >> theme;
-			chooseMap->addItem(map, theme);
+			input >> limit;
+			mapInfo.push_back(theme);
+			if (limit)
+				mapInfo.push_back(limit);
+			else
+				mapInfo.push_back(18);
+			chooseMap->addItem(map, mapInfo);
 			mapCfgFile.close();
 		}
 	}
@@ -76,12 +90,27 @@ HWMapContainer::HWMapContainer(QWidget * parent) :
 	QLabel * lblMap = new QLabel(tr("Map"), this);
 	mainLayout.addWidget(lblMap, 1, 0);
 
+	lblFilter = new QLabel(tr("Filter"), this);
+	mainLayout.addWidget(lblFilter, 2, 0);
+
+	CB_TemplateFilter = new QComboBox(this);
+    CB_TemplateFilter->addItem(tr("All"), 0);
+    CB_TemplateFilter->addItem(tr("Small"), 1);
+    CB_TemplateFilter->addItem(tr("Medium"), 2);
+    CB_TemplateFilter->addItem(tr("Large"), 3);
+    CB_TemplateFilter->addItem(tr("Cavern"), 4);
+    CB_TemplateFilter->addItem(tr("Wacky"), 5);
+	mainLayout.addWidget(CB_TemplateFilter, 2, 1);
+
+	connect(CB_TemplateFilter, SIGNAL(currentIndexChanged(int)), this, SLOT(templateFilterChanged(int)));
+
 	gbThemes = new IconedGroupBox(this);
 	gbThemes->setTitleTextPadding(60);
+	gbThemes->setContentTopPadding(6);
 	gbThemes->setTitle(tr("Themes"));
 
 	//gbThemes->setStyleSheet("padding: 0px"); // doesn't work - stylesheet is set with icon
-	mainLayout.addWidget(gbThemes, 0, 2, 2, 1);
+	mainLayout.addWidget(gbThemes, 0, 2, 3, 1);
 	
 	QVBoxLayout * gbTLayout = new QVBoxLayout(gbThemes);
 	gbTLayout->setContentsMargins(0, 0, 0 ,0);
@@ -96,7 +125,21 @@ HWMapContainer::HWMapContainer(QWidget * parent) :
 		lwThemes->addItem(lwi);
 	}
 	connect(lwThemes, SIGNAL(currentRowChanged(int)), this, SLOT(themeSelected(int)));
-	
+
+    // override default style to tighten up theme scroller
+	lwThemes->setStyleSheet(QString(
+		"QListWidget{"
+			"border: solid;"
+			"border-width: 0px;"
+			"border-radius: 0px;"
+			"border-color: transparent;"
+			"background-color: #0d0544;"
+			"color: #ffcc00;"
+			"font: bold 14px;"
+			"}"
+		)
+	);
+ 
 	gbTLayout->addWidget(lwThemes);
 	lwThemes->setSizePolicy(QSizePolicy::Maximum, QSizePolicy::Minimum);
 	
@@ -122,23 +165,31 @@ void HWMapContainer::setImage(const QImage newImage)
 	p.fillRect(QRect(0, 0, 256, 128), linearGrad);
 	p.drawPixmap(QPoint(0, 0), px);
 
-	imageButt->setIcon(pxres);
-	imageButt->setIconSize(QSize(256, 128));
+    addInfoToPreview(pxres);
 	chooseMap->setCurrentIndex(0);
 	pMap = 0;
 }
 
+void HWMapContainer::setHHLimit(int newHHLimit)
+{
+    hhLimit = newHHLimit;
+}
+
 void HWMapContainer::mapChanged(int index)
 {
 	if(!index) {
 		changeImage();
 		gbThemes->show();
+        lblFilter->show();
+        CB_TemplateFilter->show();
 		emit mapChanged("+rnd+");
-		emit themeChanged(chooseMap->itemData(0).toString());
+		emit themeChanged(chooseMap->itemData(0).toList()[0].toString());
 	} else
 	{
 		loadMap(index);
 		gbThemes->hide();
+        lblFilter->hide();
+        CB_TemplateFilter->hide();
 		emit mapChanged(chooseMap->currentText());
 	}
 }
@@ -151,38 +202,81 @@ void HWMapContainer::loadMap(int index)
 		chooseMap->setCurrentIndex(0);
 		return;
 	}
-	imageButt->setIcon(mapImage);
+
+    hhLimit = chooseMap->itemData(index).toList()[1].toInt();
+    addInfoToPreview(mapImage);
+}
+
+// Should this add text to identify map size?
+void HWMapContainer::addInfoToPreview(QPixmap image)
+{
+	QPixmap finalImage = QPixmap(image.size());
+	finalImage.fill(QColor(0, 0, 0, 0));
+	
+	QPainter p(&finalImage);
+	p.drawPixmap(image.rect(), image);
+	//p.setPen(QColor(0xf4,0x9e,0xe9));
+	p.setPen(QColor(0xff,0xcc,0x00));
+	p.setBrush(QColor(0, 0, 0));
+	p.drawRect(image.rect().width() - hhSmall.rect().width() - 28, 3, 40, 20);
+	p.setFont(QFont("MS Shell Dlg", 10));
+	p.drawText(image.rect().width() - hhSmall.rect().width() - 14 - (hhLimit > 9 ? 10 : 0), 18, QString::number(hhLimit));
+	p.drawPixmap(image.rect().width() - hhSmall.rect().width() - 5, 5, hhSmall.rect().width(), hhSmall.rect().height(), hhSmall);
+
+	imageButt->setIcon(finalImage);
+	imageButt->setIconSize(image.size());
 }
 
 void HWMapContainer::changeImage()
 {
+	if (pMap)
+	{
+		disconnect(pMap, 0, this, SLOT(setImage(const QImage)));
+		disconnect(pMap, 0, this, SLOT(setHHLimit(int)));
+		pMap = 0;
+	}
+
 	pMap = new HWMap();
 	connect(pMap, SIGNAL(ImageReceived(const QImage)), this, SLOT(setImage(const QImage)));
-	pMap->getImage(m_seed.toStdString());
+	connect(pMap, SIGNAL(HHLimitReceived(int)), this, SLOT(setHHLimit(int)));
+	pMap->getImage(m_seed.toStdString(), getTemplateFilter());
 }
 
 void HWMapContainer::themeSelected(int currentRow)
 {
 	QString theme = Themes->at(currentRow);
-	chooseMap->setItemData(0, theme);
+	QList<QVariant> mapInfo;
+	mapInfo.push_back(theme);
+	mapInfo.push_back(18);
+	chooseMap->setItemData(0, mapInfo);
 	gbThemes->setIcon(QIcon(QString("%1/Themes/%2/icon.png").arg(datadir->absolutePath()).arg(theme)));
 	emit themeChanged(theme);
 }
 
 QString HWMapContainer::getCurrentSeed() const
 {
-  return m_seed;
+	return m_seed;
 }
 
 QString HWMapContainer::getCurrentMap() const
 {
-  if(!chooseMap->currentIndex()) return QString();
-  return chooseMap->currentText();
+	if(!chooseMap->currentIndex()) return QString();
+	return chooseMap->currentText();
 }
 
 QString HWMapContainer::getCurrentTheme() const
 {
-	return chooseMap->itemData(chooseMap->currentIndex()).toString();
+	return chooseMap->itemData(chooseMap->currentIndex()).toList()[0].toString();
+}
+
+int HWMapContainer::getCurrentHHLimit() const
+{
+	return hhLimit;
+}
+
+quint32 HWMapContainer::getTemplateFilter() const
+{
+	return CB_TemplateFilter->itemData(CB_TemplateFilter->currentIndex()).toInt();
 }
 
 void HWMapContainer::resizeEvent ( QResizeEvent * event )
@@ -206,13 +300,14 @@ void HWMapContainer::setMap(const QString & map)
 	
 	int id = chooseMap->findText(map);
 	if(id > 0) {
-		chooseMap->setCurrentIndex(id);
-		loadMap(id);
 		if (pMap)
 		{
 			disconnect(pMap, 0, this, SLOT(setImage(const QImage)));
+			disconnect(pMap, 0, this, SLOT(setHHLimit(int)));
 			pMap = 0;
 		}
+		chooseMap->setCurrentIndex(id);
+		loadMap(id);
 	}
 }
 
@@ -225,9 +320,9 @@ void HWMapContainer::setTheme(const QString & theme)
 
 void HWMapContainer::setRandomSeed()
 {
-  m_seed = QUuid::createUuid().toString();
-  emit seedChanged(m_seed);
-  changeImage();
+	m_seed = QUuid::createUuid().toString();
+	emit seedChanged(m_seed);
+	changeImage();
 }
 
 void HWMapContainer::setRandomTheme()
@@ -236,3 +331,15 @@ void HWMapContainer::setRandomTheme()
 	quint32 themeNum = rand() % Themes->size();
 	lwThemes->setCurrentRow(themeNum);
 }
+
+void HWMapContainer::setTemplateFilter(int filter)
+{
+	CB_TemplateFilter->setCurrentIndex(filter);
+}
+
+void HWMapContainer::templateFilterChanged(int filter)
+{
+	emit newTemplateFilter(filter);
+	changeImage();
+}
+
