@@ -1,6 +1,7 @@
 /*
  * Hedgewars, a free turn based strategy game
  * Copyright (c) 2006-2008 Ulyanov Igor <iulyanov@gmail.com>
+ * Copyright (c) 2009 Andrey Korotaev <unC0Rr@gmail.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -16,8 +17,9 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
  */
 
-#include <QMessageBox>
 #include <QDebug>
+#include <QInputDialog>
+#include <QCryptographicHash>
 
 #include "hwconsts.h"
 #include "newnetclient.h"
@@ -26,6 +28,7 @@
 #include "game.h"
 #include "gamecfgwidget.h"
 #include "teamselect.h"
+#include "misc.h"
 
 char delimeter='\n';
 
@@ -38,11 +41,27 @@ HWNewNet::HWNewNet(GameUIConfig * config, GameCFGWidget* pGameCFGWidget, TeamSel
   loginStep(0),
   netClientState(0)
 {
+// socket stuff
 	connect(&NetSocket, SIGNAL(readyRead()), this, SLOT(ClientRead()));
 	connect(&NetSocket, SIGNAL(connected()), this, SLOT(OnConnect()));
 	connect(&NetSocket, SIGNAL(disconnected()), this, SLOT(OnDisconnect()));
 	connect(&NetSocket, SIGNAL(error(QAbstractSocket::SocketError)), this,
 			SLOT(displayError(QAbstractSocket::SocketError)));
+
+// config stuff
+	connect(this, SIGNAL(paramChanged(const QString &, const QStringList &)), pGameCFGWidget, SLOT(setParam(const QString &, const QStringList &)));
+	connect(pGameCFGWidget, SIGNAL(paramChanged(const QString &, const QStringList &)), this, SLOT(onParamChanged(const QString &, const QStringList &)));
+	connect(this, SIGNAL(configAsked()), pGameCFGWidget, SLOT(fullNetConfig()));
+}
+
+HWNewNet::~HWNewNet()
+{
+	if (m_game_connected)
+	{
+		RawSendNet(QString("QUIT%1%2").arg(delimeter).arg("User quit"));
+		emit Disconnected();
+	}
+	NetSocket.flush();
 }
 
 void HWNewNet::Connect(const QString & hostName, quint16 port, const QString & nick)
@@ -54,8 +73,9 @@ void HWNewNet::Connect(const QString & hostName, quint16 port, const QString & n
 void HWNewNet::Disconnect()
 {
 	if (m_game_connected)
-		RawSendNet(QString("QUIT"));
+		RawSendNet(QString("QUIT%1%2").arg(delimeter).arg("User quit"));
 	m_game_connected = false;
+	
 	NetSocket.disconnectFromHost();
 }
 
@@ -67,8 +87,9 @@ void HWNewNet::CreateRoom(const QString & room)
 		return;
 	}
 	
-	RawSendNet(QString("CREATE%1%2").arg(delimeter).arg(room));
+	RawSendNet(QString("CREATE_ROOM%1%2").arg(delimeter).arg(room));
 	m_pGameCFGWidget->setEnabled(true);
+	m_pTeamSelWidget->setInteractivity(true);
 	isChief = true;
 }
 
@@ -82,9 +103,9 @@ void HWNewNet::JoinRoom(const QString & room)
 	
 	loginStep++;
 
-	RawSendNet(QString("JOIN%1%2").arg(delimeter).arg(room));
+	RawSendNet(QString("JOIN_ROOM%1%2").arg(delimeter).arg(room));
 	m_pGameCFGWidget->setEnabled(false);
-	m_pTeamSelWidget->setNonInteractive();
+	m_pTeamSelWidget->setInteractivity(false);
 	isChief = false;
 }
 
@@ -95,6 +116,7 @@ void HWNewNet::AddTeam(const HWTeam & team)
 	     team.teamColor.name() + delimeter +
 	     team.Grave + delimeter +
 	     team.Fort + delimeter +
+	     team.Voicepack + delimeter +
 	     QString::number(team.difficulty);
 
 	for(int i = 0; i < 8; ++i)
@@ -121,7 +143,7 @@ void HWNewNet::SendNet(const QByteArray & buf)
 {
   QString msg = QString(buf.toBase64());
 
-  RawSendNet(QString("GAMEMSG%1%2").arg(delimeter).arg(msg));
+  RawSendNet(QString("EM%1%2").arg(delimeter).arg(msg));
 }
 
 void HWNewNet::RawSendNet(const QString & str)
@@ -131,7 +153,7 @@ void HWNewNet::RawSendNet(const QString & str)
 
 void HWNewNet::RawSendNet(const QByteArray & buf)
 {
-	//qDebug() << "Client: " << QString(buf).split("\n");
+//	qDebug() << "Client: " << QString(buf).split("\n");
 	NetSocket.write(buf);
 	NetSocket.write("\n\n", 2);
 }
@@ -151,39 +173,35 @@ void HWNewNet::ClientRead()
 
 void HWNewNet::OnConnect()
 {
-	RawSendNet(QString("NICK%1%2").arg(delimeter).arg(mynick));
-	RawSendNet(QString("PROTO%1%2").arg(delimeter).arg(*cProtoVer));
 }
 
 void HWNewNet::OnDisconnect()
 {
-  //emit ChangeInTeams(QStringList());
-  if(m_game_connected) emit Disconnected();
-  m_game_connected = false;
+	if(m_game_connected) emit Disconnected();
+	m_game_connected = false;
 }
 
 void HWNewNet::displayError(QAbstractSocket::SocketError socketError)
 {
+	emit Disconnected();
+	
 	switch (socketError) {
 		case QAbstractSocket::RemoteHostClosedError:
 			break;
 		case QAbstractSocket::HostNotFoundError:
-			QMessageBox::information(0, tr("Error"),
-					tr("The host was not found. Please check the host name and port settings."));
+			emit showMessage(tr("The host was not found. Please check the host name and port settings."));
 			break;
 		case QAbstractSocket::ConnectionRefusedError:
-			QMessageBox::information(0, tr("Error"),
-					tr("Connection refused"));
+			emit showMessage(tr("Connection refused"));
 			break;
 		default:
-			QMessageBox::information(0, tr("Error"),
-					NetSocket.errorString());
+			emit showMessage(NetSocket.errorString());
 		}
 }
 
 void HWNewNet::ParseCmd(const QStringList & lst)
 {
-	//qDebug() << "Server: " << lst;
+//	qDebug() << "Server: " << lst;
 
 	if(!lst.size())
 	{
@@ -191,37 +209,45 @@ void HWNewNet::ParseCmd(const QStringList & lst)
 		return;
 	}
 
-	if ((lst[0] == "NICK") || (lst[0] == "PROTO"))
+	if (lst[0] == "NICK")
 	{
-		loginStep++;
-		if (loginStep == 2)
-		{
-			netClientState = 2;
-			RawSendNet(QString("LIST"));
-		}
+		mynick = lst[1];
 		return ;
 	}
 
+	if (lst[0] == "PROTO")
+		return ;
+
 	if (lst[0] == "ERROR") {
 		if (lst.size() == 2)
-			QMessageBox::information(0, 0, "Error: " + lst[1]);
+			emit showMessage("Error: " + lst[1]);
 		else
-			QMessageBox::information(0, 0, "Unknown error");
+			emit showMessage("Unknown error");
 		return;
 	}
 
 	if (lst[0] == "WARNING") {
 		if (lst.size() == 2)
-			QMessageBox::information(0, 0, "Warning: " + lst[1]);
+			emit showMessage("Warning: " + lst[1]);
 		else
-			QMessageBox::information(0, 0, "Unknown warning");
+			emit showMessage("Unknown warning");
 		return;
 	}
 
 	if (lst[0] == "CONNECTED") {
+		RawSendNet(QString("NICK%1%2").arg(delimeter).arg(mynick));
+		RawSendNet(QString("PROTO%1%2").arg(delimeter).arg(*cProtoVer));
 		netClientState = 1;
 		m_game_connected = true;
-		emit Connected();
+		emit adminAccess(false);
+		return;
+	}
+
+	if (lst[0] == "PING") {
+		if (lst.size() > 1)
+			RawSendNet(QString("PONG%1%2").arg(delimeter).arg(lst[1]));
+		else
+			RawSendNet(QString("PONG"));
 		return;
 	}
 
@@ -242,37 +268,66 @@ void HWNewNet::ParseCmd(const QStringList & lst)
 		return;
 	}
 
-	if (lst[0] == "CHAT_STRING") {
+	if (lst[0] == "CHAT") {
 		if(lst.size() < 3)
 		{
-			qWarning("Net: Empty CHAT_STRING message");
+			qWarning("Net: Empty CHAT message");
 			return;
 		}
-		emit chatStringFromNet(formatChatMsg(lst[1], lst[2]));
+		if (netClientState == 2)
+			emit chatStringLobby(formatChatMsg(lst[1], lst[2]));
+		else
+			emit chatStringFromNet(formatChatMsg(lst[1], lst[2]));
+		return;
+	}
+
+	if (lst[0] == "INFO") {
+		if(lst.size() < 5)
+		{
+			qWarning("Net: Malformed INFO message");
+			return;
+		}
+		QStringList tmp = lst;
+		tmp.removeFirst();
+		if (netClientState == 2)
+			emit chatStringLobby(tmp.join("\n"));
+		else
+			emit chatStringFromNet(tmp.join("\n"));
 		return;
 	}
 
 	if (lst[0] == "READY") {
-		if(lst.size() != 2)
+		if(lst.size() < 2)
 		{
 			qWarning("Net: Malformed READY message");
 			return;
 		}
-		emit setReadyStatus(lst[1], true);
+		for(int i = 1; i < lst.size(); ++i)
+		{
+			if (lst[i] == mynick)
+				emit setMyReadyStatus(true);
+			emit setReadyStatus(lst[i], true);
+		}
 		return;
 	}
+	
 	if (lst[0] == "NOT_READY") {
-		if(lst.size() != 2)
+		if(lst.size() < 2)
 		{
 			qWarning("Net: Malformed NOT_READY message");
 			return;
 		}
-		emit setReadyStatus(lst[1], false);
+		for(int i = 1; i < lst.size(); ++i)
+		{
+			if (lst[i] == mynick)
+				emit setMyReadyStatus(false);
+			emit setReadyStatus(lst[i], false);
+		}
 		return;
 	}
 
 	if (lst[0] == "ADD_TEAM") {
-		if(lst.size() != 21)
+		if(lst.size() != 23)
 		{
 			qWarning("Net: Bad ADDTEAM message");
 			return;
@@ -290,16 +345,24 @@ void HWNewNet::ParseCmd(const QStringList & lst)
 			return;
 		}
 		m_pTeamSelWidget->removeNetTeam(HWTeam(lst[1]));
-		if (netClientState == 5) // we're in game, need to tell the engine about this
-		{
-			QByteArray em;
-			HWProto::addStringToBuffer(em, "F" + lst[1]);
-			emit FromNet(em);
-		}
 		return;
 	}
 
-	if(lst[0]=="JOINED") {
+	if(lst[0] == "ROOMABANDONED") {
+		netClientState = 2;
+		emit showMessage(HWNewNet::tr("Room destroyed"));
+		emit LeftRoom();
+		return;
+	}
+
+	if(lst[0] == "KICKED") {
+		netClientState = 2;
+		emit showMessage(HWNewNet::tr("You got kicked"));
+		emit LeftRoom();
+		return;
+	}
+
+	if(lst[0] == "JOINED") {
 		if(lst.size() < 2)
 		{
 			qWarning("Net: Bad JOINED message");
@@ -312,8 +375,9 @@ void HWNewNet::ParseCmd(const QStringList & lst)
 			{
 				netClientState = 3;
 				emit EnteredGame();
+				emit roomMaster(isChief);
 				if (isChief)
-					ConfigAsked();
+					emit configAsked();
 			}
 			emit nickAdded(lst[i]);
 			emit chatStringFromNet(QString(tr("*** %1 joined")).arg(lst[i]));
@@ -321,6 +385,28 @@ void HWNewNet::ParseCmd(const QStringList & lst)
 		return;
 	}
 
+	if(lst[0] == "LOBBY:JOINED") {
+		if(lst.size() < 2)
+		{
+			qWarning("Net: Bad JOINED message");
+			return;
+		}
+		
+		for(int i = 1; i < lst.size(); ++i)
+		{
+			if (lst[i] == mynick)
+			{
+				netClientState = 2;
+				RawSendNet(QString("LIST"));
+				emit Connected();
+			}
+
+			emit nickAddedLobby(lst[i]);
+			emit chatStringLobby(QString(tr("*** %1 joined")).arg(lst[i]));
+		}
+		return;
+	}
+
 	if(lst[0] == "LEFT") {
 		if(lst.size() < 2)
 		{
@@ -328,7 +414,34 @@ void HWNewNet::ParseCmd(const QStringList & lst)
 			return;
 		}
 		emit nickRemoved(lst[1]);
-		emit chatStringFromNet(QString(tr("*** %1 left")).arg(lst[1]));
+		if (lst.size() < 3)
+			emit chatStringFromNet(QString(tr("*** %1 left")).arg(lst[1]));
+		else
+			emit chatStringFromNet(QString(tr("*** %1 left (%2)")).arg(lst[1], lst[2]));
+		return;
+	}
+
+	if(lst[0] == "ROOM") {
+		if(lst.size() < 2)
+		{
+			qWarning("Net: Bad ROOM message");
+			return;
+		}
+		RawSendNet(QString("LIST"));
+		return;
+	}
+
+	if(lst[0] == "LOBBY:LEFT") {
+		if(lst.size() < 2)
+		{
+			qWarning("Net: Bad LOBBY:LEFT message");
+			return;
+		}
+		emit nickRemovedLobby(lst[1]);
+		if (lst.size() < 3)
+			emit chatStringLobby(QString(tr("*** %1 left")).arg(lst[1]));
+		else
+			emit chatStringLobby(QString(tr("*** %1 left (%2)")).arg(lst[1], lst[2]));
 		return;
 	}
 
@@ -338,6 +451,15 @@ void HWNewNet::ParseCmd(const QStringList & lst)
 		return;
 	}
 
+	if (lst[0] == "ASKPASSWORD") {
+		QString password = QInputDialog::getText(0, tr("Password"), tr("Enter your password:"), QLineEdit::Password);
+
+		QString hash = QCryptographicHash::hash(password.toLatin1(), QCryptographicHash::Md5).toHex();
+
+		RawSendNet(QString("PASSWORD%1%2").arg(delimeter).arg(hash));
+		return;
+	}
+
 	if (lst[0] == "TEAM_ACCEPTED") {
 		if (lst.size() != 2)
 		{
@@ -348,53 +470,20 @@ void HWNewNet::ParseCmd(const QStringList & lst)
 		return;
 	}
 
-	if (lst[0] == "MAP") {
-		if (lst.size() != 2)
-		{
-			qWarning("Net: Bad MAP message");
-			return;
-		}
-		emit mapChanged(lst[1]);
-		return;
-	}
 
-
-	if (lst[0] == "CONFIG_PARAM") {
+	if (lst[0] == "CFG") {
 		if(lst.size() < 3)
 		{
-			qWarning("Net: Bad CONFIG_PARAM message");
+			qWarning("Net: Bad CFG message");
 			return;
 		}
-		if (lst[1] == "SEED") {
-			emit seedChanged(lst[2]);
-			return;
-		}
-		if (lst[1] == "THEME") {
-			emit themeChanged(lst[2]);
-			return;
-		}
-		if (lst[1] == "HEALTH") {
-			emit initHealthChanged(lst[2].toUInt());
-			return;
-		}
-		if (lst[1] == "TURNTIME") {
-			emit turnTimeChanged(lst[2].toUInt());
-			return;
-		}
-		if (lst[1] == "FORTSMODE") {
-			emit fortsModeChanged(lst[2].toInt() != 0);
-			return;
-		}
-		if (lst[1] == "DIVIDETEAMS") {
-			emit teamsDivideChanged(lst[2].toInt() != 0);
-			return;
-		}
-		if (lst[1] == "AMMO") {
-			if(lst.size() < 4) return;
-			emit ammoChanged(lst[3], lst[2]);
-			return;
-		}
-		qWarning() << "Net: Unknown 'CONFIG_PARAM' message:" << lst;
+		QStringList tmp = lst;
+		tmp.removeFirst();
+		tmp.removeFirst();
+		if (lst[1] == "SCHEME")
+			emit netSchemeConfig(tmp);
+		else
+			emit paramChanged(lst[1], tmp);
 		return;
 	}
 
@@ -422,36 +511,39 @@ void HWNewNet::ParseCmd(const QStringList & lst)
 		return;
 	}
 
-	if (lst[0] == "GAMEMSG") {
+	if (lst[0] == "EM") {
 		if(lst.size() < 2)
 		{
-			qWarning("Net: Bad GAMEMSG message");
+			qWarning("Net: Bad EM message");
 			return;
 		}
-		QByteArray em = QByteArray::fromBase64(lst[1].toAscii());
-		emit FromNet(em);
+		for(int i = 1; i < lst.size(); ++i)
+		{
+			QByteArray em = QByteArray::fromBase64(lst[i].toAscii());
+			emit FromNet(em);
+		}
+		return;
+	}
+
+	if (lst[0] == "BYE") {
+		if (lst.size() < 2)
+		{
+			qWarning("Net: Bad BYE message");
+			return;
+		}
+		emit showMessage(HWNewNet::tr("Quit reason: ") + lst[1]);
+		return;
+	}
+
+
+	if (lst[0] == "ADMIN_ACCESS") {
+		emit adminAccess(true);
 		return;
 	}
 
 	qWarning() << "Net: Unknown message:" << lst;
 }
 
-
-void HWNewNet::ConfigAsked()
-{
-	QString map = m_pGameCFGWidget->getCurrentMap();
-	if (map.size())
-		onMapChanged(map);
-
-	onSeedChanged(m_pGameCFGWidget->getCurrentSeed());
-	onThemeChanged(m_pGameCFGWidget->getCurrentTheme());
-	onInitHealthChanged(m_pGameCFGWidget->getInitHealth());
-	onTurnTimeChanged(m_pGameCFGWidget->getTurnTime());
-	onFortsModeChanged(m_pGameCFGWidget->getGameFlags() & 0x1);
-	// always initialize with default ammo (also avoiding complicated cross-class dependencies)
-	onWeaponsNameChanged("Default", cDefaultAmmoStore->mid(10));
-}
-
 void HWNewNet::RunGame()
 {
 	emit AskForRunGame();
@@ -475,52 +567,31 @@ void HWNewNet::onTeamColorChanged(const HWTeam& team)
 			.arg(team.teamColor.name()));
 }
 
-void HWNewNet::onSeedChanged(const QString & seed)
+void HWNewNet::onParamChanged(const QString & param, const QStringList & value)
 {
-  if (isChief) RawSendNet(QString("CONFIG_PARAM%1SEED%1%2").arg(delimeter).arg(seed));
-}
-
-void HWNewNet::onMapChanged(const QString & map)
-{
-  if (isChief) RawSendNet(QString("MAP%1%2").arg(delimeter).arg(map));
-}
-
-void HWNewNet::onThemeChanged(const QString & theme)
-{
-  if (isChief) RawSendNet(QString("CONFIG_PARAM%1THEME%1%2").arg(delimeter).arg(theme));
-}
-
-void HWNewNet::onInitHealthChanged(int health)
-{
-  if (isChief) RawSendNet(QString("CONFIG_PARAM%1HEALTH%1%2").arg(delimeter).arg(health));
-}
-
-void HWNewNet::onTurnTimeChanged(int time)
-{
-  if (isChief) RawSendNet(QString("CONFIG_PARAM%1TURNTIME%1%2").arg(delimeter).arg(time));
-}
-
-void HWNewNet::onFortsModeChanged(bool value)
-{
-  if (isChief) RawSendNet(QString("CONFIG_PARAM%1FORTSMODE%1%2").arg(delimeter).arg(value));
-}
-
-void HWNewNet::onTeamsDivideChanged(bool value)
-{
-  if (isChief) RawSendNet(QString("CONFIG_PARAM%1DIVIDETEAMS%1%2").arg(delimeter).arg(value));
-}
-
-void HWNewNet::onWeaponsNameChanged(const QString& name, const QString& ammo)
-{
-  if (isChief) RawSendNet(QString("CONFIG_PARAM%1AMMO%1%2%1%3").arg(delimeter).arg(ammo).arg(name));
+	if (isChief)
+		RawSendNet(
+				QString("CFG%1%2%1%3")
+					.arg(delimeter)
+					.arg(param)
+					.arg(value.join(QString(delimeter)))
+				);
 }
 
 void HWNewNet::chatLineToNet(const QString& str)
 {
-  if(str!="") {
-    RawSendNet(QString("CHAT_STRING")+delimeter+str);
-    emit(chatStringFromMe(formatChatMsg(mynick, str)));
-  }
+	if(str != "") {
+		RawSendNet(QString("CHAT") + delimeter + str);
+		emit(chatStringFromMe(formatChatMsg(mynick, str)));
+	}
+}
+
+void HWNewNet::chatLineToLobby(const QString& str)
+{
+	if(str != "") {
+		RawSendNet(QString("CHAT") + delimeter + str);
+		emit(chatStringFromMeLobby(formatChatMsg(mynick, str)));
+	}
 }
 
 void HWNewNet::askRoomsList()
@@ -552,11 +623,21 @@ QString HWNewNet::formatChatMsg(const QString & nick, const QString & msg)
 		return QString("%1: %2").arg(nick).arg(msg);
 }
 
+void HWNewNet::banPlayer(const QString & nick)
+{
+	RawSendNet(QString("BAN%1%2").arg(delimeter).arg(nick));
+}
+
 void HWNewNet::kickPlayer(const QString & nick)
 {
 	RawSendNet(QString("KICK%1%2").arg(delimeter).arg(nick));
 }
 
+void HWNewNet::infoPlayer(const QString & nick)
+{
+	RawSendNet(QString("INFO%1%2").arg(delimeter).arg(nick));
+}
+
 void HWNewNet::startGame()
 {
 	RawSendNet(QString("START_GAME"));
@@ -571,3 +652,19 @@ void HWNewNet::toggleRestrictTeamAdds()
 {
 	RawSendNet(QString("TOGGLE_RESTRICT_TEAMS"));
 }
+
+void HWNewNet::partRoom()
+{
+	netClientState = 2;
+	RawSendNet(QString("PART"));
+}
+
+bool HWNewNet::isInRoom()
+{
+	return netClientState > 2;
+}
+
+void HWNewNet::newServerMessage(const QString & msg)
+{
+	RawSendNet(QString("SET_SERVER_MESSAGE%1%2").arg(delimeter).arg(msg));
+}
