@@ -39,9 +39,11 @@ class HWNewNet : public QObject
 
  public:
   HWNewNet(GameUIConfig * config, GameCFGWidget* pGameCFGWidget, TeamSelWidget* pTeamSelWidget);
+  ~HWNewNet();
   void Connect(const QString & hostName, quint16 port, const QString & nick);
   void Disconnect();
   bool isRoomChief();
+  bool isInRoom();
 
  private:
   GameUIConfig* config;
@@ -54,7 +56,6 @@ class HWNewNet : public QObject
   QString seed;
   bool m_game_connected;
 
-  void ConfigAsked();
   void RunGame();
 
   template <typename T>
@@ -91,53 +92,59 @@ class HWNewNet : public QObject
   void Connected();
   void Disconnected();
   void EnteredGame();
+  void LeftRoom();
   void nickAdded(const QString& nick);
   void nickRemoved(const QString& nick);
+  void nickAddedLobby(const QString& nick);
+  void nickRemovedLobby(const QString& nick);
   void FromNet(const QByteArray & buf);
-  void AddNetTeam(const HWTeam&);
+  void adminAccess(bool);
+  void roomMaster(bool);
 
-  void seedChanged(const QString & seed);
-  void mapChanged(const QString & map);
-  void themeChanged(const QString & theme);
-  void initHealthChanged(int health);
-  void turnTimeChanged(int time);
-  void fortsModeChanged(bool value);
-  void teamsDivideChanged(bool value);
+  void netSchemeConfig(QStringList &);
+  void paramChanged(const QString & param, const QStringList & value);
+  void configAsked();
+
+  void AddNetTeam(const HWTeam&);
   void hhnumChanged(const HWTeam&);
   void teamColorChanged(const HWTeam&);
+  void chatStringLobby(const QString&);
   void chatStringFromNet(const QString&);
   void chatStringFromMe(const QString&);
-  void ammoChanged(const QString& name, const QString& ammo);
+  void chatStringFromMeLobby(const QString&);
 
   void roomsList(const QStringList&);
   void serverMessage(const QString &);
 
   void setReadyStatus(const QString & nick, bool isReady);
+  void setMyReadyStatus(bool isReady);
+  void showMessage(const QString &);
 
  public slots:
   void ToggleReady();
   void chatLineToNet(const QString& str);
+  void chatLineToLobby(const QString& str);
   void SendNet(const QByteArray & buf);
   void AddTeam(const HWTeam & team);
   void RemoveTeam(const HWTeam& team);
-  void onSeedChanged(const QString & seed);
-  void onMapChanged(const QString & map);
-  void onThemeChanged(const QString & theme);
-  void onInitHealthChanged(int health);
-  void onTurnTimeChanged(int time);
-  void onFortsModeChanged(bool value);
-  void onTeamsDivideChanged(bool value);
   void onHedgehogsNumChanged(const HWTeam& team);
   void onTeamColorChanged(const HWTeam& team);
-  void onWeaponsNameChanged(const QString& name, const QString& ammo);
+  void onParamChanged(const QString & param, const QStringList & value);
+  
+  void newServerMessage(const QString &);
+
+
   void JoinRoom(const QString & room);
   void CreateRoom(const QString & room);
   void askRoomsList();
   void gameFinished();
+  void banPlayer(const QString &);
   void kickPlayer(const QString &);
+  void infoPlayer(const QString &);
   void startGame();
   void toggleRestrictJoins();
   void toggleRestrictTeamAdds();
+  void partRoom();
 
  private slots:
   void ClientRead();
