@@ -23,9 +23,11 @@
 #include <QPushButton>
 #include <QFont>
 #include <QGridLayout>
+#include <QSignalMapper>
 
 #include "binds.h"
 #include "mapContainer.h"
+#include "togglebutton.h"
 
 class QPushButton;
 class QGroupBox;
@@ -42,6 +44,8 @@ class QTableView;
 class QTextBrowser;
 class QTableWidget;
 class QAction;
+class QDataWidgetMapper;
+class QAbstractItemModel;
 
 class GameCFGWidget;
 class TeamSelWidget;
@@ -52,9 +56,12 @@ class FPSEdit;
 class HWChatWidget;
 class SelWeaponWidget;
 class IconedGroupBox;
+class FreqSpinBox;
 
 class AbstractPage : public QWidget
 {
+	Q_OBJECT
+
  public:
 
  protected:
@@ -141,6 +148,7 @@ class PageEditTeam : public AbstractPage
 
 public:
 	PageEditTeam(QWidget* parent = 0);
+	QSignalMapper* signalMapper;
 	QGroupBox *GBoxHedgehogs;
 	QGroupBox *GBoxTeam;
 	QGroupBox *GBoxFort;
@@ -148,6 +156,7 @@ public:
 	SquareLabel *FortPreview;
 	QComboBox *CBGrave;
 	QComboBox *CBTeamLvl;
+	QComboBox *CBVoicepack;
 	QGroupBox *GBoxBinds;
 	QToolBox *BindsBox;
 	QWidget *page_A;
@@ -156,16 +165,22 @@ public:
 	QWidget *page_O;
 	QPushButton *BtnTeamDiscard;
 	QPushButton *BtnTeamSave;
+	QPushButton * BtnTestSound;
 	QLineEdit * TeamNameEdit;
 	QLineEdit * HHNameEdit[8];
 	QComboBox * HHHats[8];
+	QPushButton * randButton[8];
 	QComboBox * CBBind[BINDS_NUMBER];
+	QPushButton * randTeamButton;
 
 public slots:
 	void CBFort_activated(const QString & gravename);
 
 private:
 	QLabel * LBind[BINDS_NUMBER];
+
+private slots:
+	void testSound();
 };
 
 class PageMultiplayer : public AbstractPage
@@ -205,10 +220,14 @@ public:
 	QCheckBox *CBFrontendFullscreen;
 	QCheckBox *CBShowFPS;
 	QCheckBox *CBAltDamage;
+	QCheckBox *CBNameWithDate;
 	FPSEdit *fpsedit;
 	QPushButton *BtnSaveOptions;
 	QLabel *labelNN;
+	QSpinBox * volumeBox;
 	QLineEdit *editNetNick;
+	QCheckBox *CBReduceQuality;
+	QCheckBox *CBFrontendEffects;
 };
 
 class PageNet : public AbstractPage
@@ -224,7 +243,6 @@ public:
 	QPushButton * BtnNetConnect;
 	QPushButton * BtnNetSvrStart;
 	QPushButton * BtnSpecifyServer;
-	QPushButton * BtnOfficialServer;
 
 private:
 	QGroupBox * ConnGroupBox;
@@ -269,8 +287,8 @@ public:
 	QPushButton *BtnBack;
 	QPushButton *BtnGo;
 	QPushButton *BtnMaster;
+	QPushButton *BtnStart;
 	
-	QAction * startGame;
 	QAction * restrictJoins;
 	QAction * restrictTeamAdds;
 
@@ -278,6 +296,10 @@ public:
 
 	TeamSelWidget* pNetTeamsWidget;
 	GameCFGWidget* pGameCFG;
+
+public slots:
+	void setReadyStatus(bool isReady);
+	void setMasterMode(bool isMaster);
 };
 
 class PageInfo : public AbstractPage
@@ -291,17 +313,6 @@ public:
 	About *about;
 };
 
-class PageGameStats : public AbstractPage
-{
-	Q_OBJECT
-
-public:
-	PageGameStats(QWidget* parent = 0);
-
-	QPushButton *BtnBack;
-	QLabel *labelGameStats;
-};
-
 class PageSinglePlayer : public AbstractPage
 {
 	Q_OBJECT
@@ -364,10 +375,12 @@ public:
 	QPushButton * BtnCreate;
 	QPushButton * BtnJoin;
 	QPushButton * BtnRefresh;
-	QTextBrowser * serverMessage;
+	QPushButton * BtnAdmin;
+	HWChatWidget * chatWidget;
 	
 public slots:
 	void setRoomsList(const QStringList & list);
+	void setAdmin(bool);
 
 private slots:
 	void onCreateClick();
@@ -380,4 +393,95 @@ signals:
 	void askForRoomList();
 };
 
+class PageConnecting : public AbstractPage
+{
+	Q_OBJECT
+
+public:
+	PageConnecting(QWidget* parent = 0);
+};
+
+class PageScheme : public AbstractPage
+{
+	Q_OBJECT
+
+public:
+	PageScheme(QWidget* parent = 0);
+
+	QPushButton * BtnBack;
+	QPushButton * BtnNew;
+	QPushButton * BtnDelete;
+	QPushButton * BtnSave;
+
+	void setModel(QAbstractItemModel * model);
+
+private:
+	QDataWidgetMapper * mapper;
+	ToggleButtonWidget * TBW_mode_Forts;
+	ToggleButtonWidget * TBW_teamsDivide;
+	ToggleButtonWidget * TBW_solid;
+	ToggleButtonWidget * TBW_border;
+	ToggleButtonWidget * TBW_lowGravity;
+	ToggleButtonWidget * TBW_laserSight;
+	ToggleButtonWidget * TBW_invulnerable;
+	ToggleButtonWidget * TBW_mines;
+	ToggleButtonWidget * TBW_vampiric;
+	ToggleButtonWidget * TBW_karma;
+	ToggleButtonWidget * TBW_artillery;
+
+	QSpinBox * SB_DamageModifier;
+	QSpinBox * SB_TurnTime;
+	QSpinBox * SB_InitHealth;
+	QSpinBox * SB_SuddenDeath;
+	FreqSpinBox * SB_CaseProb;
+	QSpinBox * SB_MinesTime;
+	QSpinBox * SB_Mines;
+	QLineEdit * LE_name;
+	QComboBox * selectScheme;
+
+	QGroupBox * gbGameModes;
+	QGroupBox * gbBasicSettings;
+
+private slots:
+	void newRow();
+	void deleteRow();
+	void schemeSelected(int);
+};
+
+class PageAdmin : public AbstractPage
+{
+	Q_OBJECT
+
+public:
+	PageAdmin(QWidget* parent = 0);
+
+	QPushButton * BtnBack;
+
+private:
+	QLineEdit * leServerMessage;
+	QPushButton * pbSetSM;
+
+private slots:
+	void smChanged();
+
+public slots:
+	void serverMessage(const QString & str);
+
+signals:
+	void setServerMessage(const QString & str);
+};
+
+
+class PageNetType : public AbstractPage
+{
+	Q_OBJECT
+
+public:
+	PageNetType(QWidget* parent = 0);
+
+	QPushButton * BtnBack;
+	QPushButton * BtnLAN;
+	QPushButton * BtnOfficialServer;
+};
+
 #endif // PAGES_H
