@@ -1,6 +1,6 @@
 /*
  * Hedgewars, a free turn based strategy game
- * Copyright (c) 2005-2007 Andrey Korotaev <unC0Rr@gmail.com>
+ * Copyright (c) 2005-2007, 2009 Andrey Korotaev <unC0Rr@gmail.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -23,7 +23,6 @@
 #include <QLineEdit>
 #include "team.h"
 #include "hwform.h"
-#include "predefteams.h"
 #include "pages.h"
 #include "hwconsts.h"
 #include "hats.h"
@@ -40,8 +39,9 @@ HWTeam::HWTeam(const QString & teamname) :
 		HHName[i].sprintf("hedgehog %d", i);
 		HHHat[i] = "NoHat";
 	}
-	Grave = "Simple";
-	Fort = "Barrelhouse";
+	Grave = "Statue";
+	Fort = "Plane";
+	Voicepack = "Default";
 	for(int i = 0; i < BINDS_NUMBER; i++)
 	{
 		binds[i].action = cbinds[i].action;
@@ -54,44 +54,35 @@ HWTeam::HWTeam(const QStringList& strLst) :
   m_isNetTeam(true)
 {
 	// net teams are configured from QStringList
-	if(strLst.size() < 20) throw HWTeamConstructException();
+	if(strLst.size() != 22) throw HWTeamConstructException();
 	TeamName = strLst[0];
 	Grave = strLst[1];
 	Fort = strLst[2];
-	difficulty = strLst[3].toUInt();
+	Voicepack = strLst[3];
+	Owner = strLst[4];
+	difficulty = strLst[5].toUInt();
 	for(int i = 0; i < 8; i++)
 	{
-		HHName[i]=strLst[i * 2 + 4];
-		HHHat[i]=strLst[i * 2 + 5];
+		HHName[i]=strLst[i * 2 + 6];
+		HHHat[i]=strLst[i * 2 + 7];
 	}
 }
 
-HWTeam::HWTeam(quint8 num) :
+HWTeam::HWTeam() :
   difficulty(0),
   numHedgehogs(4),
   m_isNetTeam(false)
 {
-	num %= PREDEFTEAMS_COUNT;
-	TeamName = QApplication::translate("teams", pteams[num].TeamName);
-	HHName[0] = QApplication::translate("teams", pteams[num].hh0name);
-	HHName[1] = QApplication::translate("teams", pteams[num].hh1name);
-	HHName[2] = QApplication::translate("teams", pteams[num].hh2name);
-	HHName[3] = QApplication::translate("teams", pteams[num].hh3name);
-	HHName[4] = QApplication::translate("teams", pteams[num].hh4name);
-	HHName[5] = QApplication::translate("teams", pteams[num].hh5name);
-	HHName[6] = QApplication::translate("teams", pteams[num].hh6name);
-	HHName[7] = QApplication::translate("teams", pteams[num].hh7name);
-	HHHat[0] = pteams[num].hh0hat;
-	HHHat[1] = pteams[num].hh1hat;
-	HHHat[2] = pteams[num].hh2hat;
-	HHHat[3] = pteams[num].hh3hat;
-	HHHat[4] = pteams[num].hh4hat;
-	HHHat[5] = pteams[num].hh5hat;
-	HHHat[6] = pteams[num].hh6hat;
-	HHHat[7] = pteams[num].hh7hat;
-	
-	Grave = pteams[num].Grave;
-	Fort = pteams[num].Fort;
+	TeamName = QString("Team");
+	for (int i = 0; i < 8; i++)
+	{
+		HHName[i].sprintf("hedgehog %d", i);
+		HHHat[i] = "NoHat";
+	}
+
+	Grave = QString("Simple"); // default
+	Fort = QString("Island"); // default
+
 	for(int i = 0; i < BINDS_NUMBER; i++)
 	{
 		binds[i].action = cbinds[i].action;
@@ -145,6 +136,11 @@ bool HWTeam::LoadFromFile()
 			str.remove(0, 5);
 			Fort = str;
 		} else
+		if (str.startsWith("voicepack "))
+		{
+			str.remove(0, 10);
+			Voicepack = str;
+		} else
 		if (str.startsWith("bind "))
 		{
 			str.remove(0, 5);
@@ -190,6 +186,7 @@ bool HWTeam::SaveToFile()
 	}
 	stream << "grave " << Grave << endl;
 	stream << "fort " << Fort << endl;
+	stream << "voicepack " << Voicepack << endl;
 	for(int i = 0; i < BINDS_NUMBER; i++)
 	{
 		stream << "bind " << binds[i].strbind << " " << binds[i].action << endl;
@@ -211,6 +208,7 @@ void HWTeam::SetToPage(HWForm * hwform)
 	hwform->ui.pageEditTeam->CBGrave->setCurrentIndex(hwform->ui.pageEditTeam->CBGrave->findText(Grave));
 
 	hwform->ui.pageEditTeam->CBFort->setCurrentIndex(hwform->ui.pageEditTeam->CBFort->findText(Fort));
+	hwform->ui.pageEditTeam->CBVoicepack->setCurrentIndex(hwform->ui.pageEditTeam->CBVoicepack->findText(Voicepack));
 	//hwform->ui.pageEditTeam->CBFort_activated(Fort);
 
 	for(int i = 0; i < BINDS_NUMBER; i++)
@@ -231,6 +229,7 @@ void HWTeam::GetFromPage(HWForm * hwform)
 
 	Grave = hwform->ui.pageEditTeam->CBGrave->currentText();
 	Fort = hwform->ui.pageEditTeam->CBFort->currentText();
+	Voicepack = hwform->ui.pageEditTeam->CBVoicepack->currentText();
 	for(int i = 0; i < BINDS_NUMBER; i++)
 	{
 		binds[i].strbind = hwform->ui.pageEditTeam->CBBind[i]->currentText();
@@ -247,6 +246,7 @@ QStringList HWTeam::TeamGameConfig(quint32 InitHealth) const
 
 	sl.push_back(QString("egrave " + Grave));
 	sl.push_back(QString("efort " + Fort));
+	sl.push_back(QString("evoicepack " + Voicepack));
 
 	if (!m_isNetTeam)
 		for(int i = 0; i < BINDS_NUMBER; i++)
@@ -277,3 +277,5 @@ bool HWTeam::operator==(const HWTeam& t1) const {
 bool HWTeam::operator<(const HWTeam& t1) const {
   return TeamName<t1.TeamName; // if names are equal - test if it is net team
 }
+
+
