@@ -50,6 +50,7 @@ TeamShowWidget::TeamShowWidget(HWTeam team, bool isPlaying, QWidget * parent) :
 
 	butt = new QPushButton(difficultyIcon, team.TeamName, this);
 	butt->setFlat(true);
+	butt->setToolTip(team.Owner);
 	mainLayout.addWidget(butt);
 	butt->setStyleSheet("QPushButton{"
 			"icon-size: 48px;"
@@ -82,14 +83,14 @@ TeamShowWidget::TeamShowWidget(HWTeam team, bool isPlaying, QWidget * parent) :
 	//QObject::connect(bText, SIGNAL(clicked()), this, SLOT(activateTeam()));
 }
 
-void TeamShowWidget::setNonInteractive()
+void TeamShowWidget::setInteractivity(bool interactive)
 {
-  if(m_team.isNetTeam()) {
-    disconnect(butt, SIGNAL(clicked()));
-   // disconnect(bText, SIGNAL(clicked()));
-  }
-  disconnect(colorButt, SIGNAL(clicked()), this, SLOT(changeTeamColor()));
-  phhoger->setNonInteractive();
+	if(m_team.isNetTeam()) {
+		butt->setEnabled(interactive);
+	}
+	
+	colorButt->setEnabled(interactive);
+	phhoger->setEnabled(interactive);
 }
 
 void TeamShowWidget::setHHNum(unsigned int num)
